"use strict";

exports.__esModule = true;
exports.default = void 0;

var _appPermissions = require("@mainframe/app-permissions");

var _utilsId = require("@mainframe/utils-id");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const DEFAULT_SETTINGS = {
  permissionsSettings: {
    grants: {
      WEB_REQUEST: {
        denied: [],
        granted: []
      }
    },
    permissionsChecked: false
  },
  walletSettings: {
    defaultEthAccount: null
  },
  approvedContacts: {}
};

class AbstractApp {
  constructor(params) {
    _defineProperty(this, "_appID", void 0);

    _defineProperty(this, "_settings", void 0);

    this._appID = params.appID;
    this._settings = params.settings == null ? {} : params.settings;
  } // Accessors


  get id() {
    return this._appID;
  }

  get settings() {
    return this._settings;
  }

  get userIDs() {
    return Object.keys(this._settings).map(_utilsId.idType);
  }

  removeUser(userID) {
    delete this._settings[userID];
  } // Settings


  getSettings(userID) {
    return this._settings[(0, _utilsId.idType)(userID)] || JSON.parse(JSON.stringify(DEFAULT_SETTINGS));
  }

  setSettings(userID, settings) {
    settings.permissionsSettings.grants = (0, _appPermissions.createStrictPermissionGrants)(settings.permissionsSettings.grants);
    this._settings[userID] = settings;
  }

  getPermissions(userID) {
    return this.getSettings(userID).permissionsSettings.grants;
  }

  setPermissions(userID, permissions) {
    const settings = this.getSettings(userID);
    settings.permissionsSettings.grants = (0, _appPermissions.createStrictPermissionGrants)(permissions);
    this._settings[userID] = settings;
  }

  setPermissionsSettings(userID, settings) {
    const appSettings = this.getSettings(userID);
    appSettings.permissionsSettings.grants = (0, _appPermissions.createStrictPermissionGrants)(settings.grants);
    appSettings.permissionsSettings.permissionsChecked = settings.permissionsChecked;
    this._settings[userID] = appSettings;
  }

  getPermission(userID, key) {
    const permissions = this.getPermissions(userID);

    if (permissions != null) {
      return permissions[key];
    }
  }

  setPermission(userID, key, value) {
    const {
      permissionsSettings
    } = this.getSettings(userID);

    if (key === 'WEB_REQUEST' && typeof value === 'object' || key !== 'WEB_REQUEST' && typeof value === 'boolean') {
      // $FlowFixMe: value polymorphism
      permissionsSettings.grants[key] = value;
      this.setPermissions(userID, permissionsSettings.grants);
    }
  }

  setPermissionsChecked(userID, checked) {
    const settings = this.getSettings(userID);
    settings.permissionsSettings.permissionsChecked = checked;
    this._settings[userID] = settings;
  }

  setDefaultEthAccount(userID, walletID, account) {
    const settings = this.getSettings(userID);
    settings.walletSettings.defaultEthAccount = account;
    this._settings[userID] = settings;
  }

  getDefaultEthAccount(userID) {
    const settings = this.getSettings(userID);

    if (settings.walletSettings) {
      return settings.walletSettings.defaultEthAccount;
    }
  }

  getLocalIDByApprovedID(userID, id) {
    const found = this.getSettings(userID).approvedContacts[id];
    return found ? found.localID : null;
  }

  getApprovedIDByLocalID(userID, localID) {
    const {
      approvedContacts
    } = this.getSettings(userID);
    const found = Object.values(approvedContacts).find( // $FlowFixMe: Object.values
    c => c.localID === localID); // $FlowFixMe: Object.values

    return found ? found.id : null;
  }

  getApprovedContacts(userID) {
    const {
      approvedContacts
    } = this.getSettings(userID);
    return approvedContacts;
  }

  approveContacts(userID, contacts) {
    const settings = this.getSettings(userID);
    const approvedContacts = settings.approvedContacts;
    const contactsAdded = {};
    contacts.forEach(c => {
      let id = this.getApprovedIDByLocalID(userID, c.localID);

      if (!id) {
        id = (0, _utilsId.uniqueID)();
        approvedContacts[id] = {
          id,
          localID: c.localID,
          publicDataOnly: c.publicDataOnly
        };
      }

      contactsAdded[id] = approvedContacts[id];
    });
    this._settings[userID] = settings;
    return contactsAdded;
  } // Session


  createSession(_userID) {
    throw new Error('Must be implemented');
  }

}

exports.default = AbstractApp;

_defineProperty(AbstractApp, "toJSON", app => ({
  appID: app._appID,
  settings: app._settings
}));