"use strict";

exports.__esModule = true;
exports.default = void 0;

var _appPermissions = require("@mainframe/app-permissions");

var _utilsId = require("@mainframe/utils-id");

var _AbstractApp = _interopRequireDefault(require("./AbstractApp"));

var _Session = _interopRequireDefault(require("./Session"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class App extends _AbstractApp.default {
  // $FlowFixMe: App type
  constructor(params) {
    super(params);

    _defineProperty(this, "_installationState", void 0);

    _defineProperty(this, "_manifest", void 0);

    this._installationState = params.installationState;
    this._manifest = params.manifest;
  } // Accessors


  get installationState() {
    return this._installationState;
  }

  set installationState(value) {
    this._installationState = value;
  }

  get manifest() {
    return this._manifest;
  }

  get mfid() {
    return this.manifest.id;
  }

  get updateFeedHash() {
    return this.manifest.updateHash;
  }

  getPermissionsChecked(userID) {
    if (!(0, _appPermissions.havePermissionsToGrant)(this._manifest.permissions)) {
      return true;
    }

    return this.getSettings(userID).permissionsSettings.permissionsChecked;
  } // Session


  createSession(userID) {
    if (!this.getPermissionsChecked(userID)) {
      throw new Error('Permissions need to be checked by user');
    }

    const requiredPermissions = this.manifest.permissions.required;

    const allManifestPermissions = _objectSpread({}, this.manifest.permissions.required, this.manifest.permissions.optional);

    const appPermissions = _objectSpread({}, requiredPermissions, {
      WEB_REQUEST: (0, _appPermissions.createWebRequestGrant)(requiredPermissions.WEB_REQUEST)
    });

    const userPermissions = this.getPermissions(userID) || {
      WEB_REQUEST: (0, _appPermissions.createWebRequestGrant)()
    };

    _appPermissions.PERMISSION_KEYS_BOOLEAN.forEach(key => {
      // Any keys not present in the manifest we set to false (deny)
      if (!allManifestPermissions[key]) {
        userPermissions[key] = false;
      }
    });

    const permissions = (0, _appPermissions.mergeGrantsToDetails)(appPermissions, userPermissions);
    const session = new _Session.default(this._appID, userID, permissions.session);
    return {
      permissions,
      sessID: (0, _utilsId.uniqueID)(),
      session
    };
  }

}

exports.default = App;

_defineProperty(App, "fromJSON", params => new App(params));

_defineProperty(App, "toJSON", app => ({
  appID: app._appID,
  installationState: app._installationState,
  manifest: app._manifest,
  settings: app._settings
}));