"use strict";

exports.__esModule = true;
exports.default = exports.getContentsPath = void 0;

var _appManifest = require("@mainframe/app-manifest");

var _appPermissions = require("@mainframe/app-permissions");

var _config = require("@mainframe/config");

var _dataTypes = require("@mainframe/data-types");

var _utilsId = require("@mainframe/utils-id");

var _utils = require("../utils");

var _App = _interopRequireDefault(require("./App"));

var _OwnApp = _interopRequireDefault(require("./OwnApp"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const appsToJSON = (0, _utils.mapObject)(_App.default.toJSON);
const appsFromJSON = (0, _utils.mapObject)(_App.default.fromJSON);
const updatesToJSON = (0, _utils.mapObject)(update => ({
  app: _App.default.toJSON(update.app),
  hasRequiredPermissionsChanges: update.hasRequiredPermissionsChanges
}));
const updatesFromJSON = (0, _utils.mapObject)(serialized => ({
  app: _App.default.fromJSON(serialized.app),
  hasRequiredPermissionsChanges: serialized.hasRequiredPermissionsChanges
}));
const ownAppsToJSON = (0, _utils.mapObject)(_OwnApp.default.toJSON);
const ownAppsFromJSON = (0, _utils.mapObject)(_OwnApp.default.fromJSON);

const getContentsPath = (env, manifest) => {
  return (0, _config.getAppContentsPath)(env, manifest.id, manifest.version);
};

exports.getContentsPath = getContentsPath;

class AppsRepository {
  constructor(params = {}) {
    _defineProperty(this, "_apps", void 0);

    _defineProperty(this, "_updates", void 0);

    _defineProperty(this, "_ownApps", void 0);

    _defineProperty(this, "_byHash", {});

    _defineProperty(this, "_byMFID", {});

    _defineProperty(this, "_appsByUser", {});

    this._apps = params.apps || {};
    this._updates = params.updates || {};
    this._ownApps = params.ownApps || {}; // Fill references for apps

    Object.keys(this._ownApps).forEach(appID => {
      const id = (0, _utilsId.idType)(appID);
      const ownApp = this._ownApps[id];

      const mfid = _dataTypes.MFID.canonical(ownApp.data.mfid);

      this._byMFID[mfid] = id;

      if (ownApp.updateFeed.feedHash) {
        this._byHash[ownApp.updateFeed.feedHash] = id;
      }

      Object.keys(ownApp.settings).forEach(uid => {
        const userID = (0, _utilsId.idType)(uid);

        if (this._appsByUser[userID]) {
          this._appsByUser[userID].push(id);
        } else {
          this._appsByUser[userID] = [id];
        }
      });
    });
    Object.keys(this._apps).forEach(appID => {
      const id = (0, _utilsId.idType)(appID);
      const app = this._apps[id];
      this._byHash[app.manifest.updateHash] = id;

      const mfid = _dataTypes.MFID.canonical(app.manifest.id);

      this._byMFID[mfid] = id;
      Object.keys(app.settings).forEach(uid => {
        const userID = (0, _utilsId.idType)(uid);

        if (this._appsByUser[userID]) {
          this._appsByUser[userID].push(id);
        } else {
          this._appsByUser[userID] = [id];
        }
      });
    });
  } // Getters


  get apps() {
    return this._apps;
  }

  get updates() {
    return this._updates;
  }

  get ownApps() {
    return this._ownApps;
  }

  getByID(id) {
    return this._apps[id] || this._ownApps[id];
  }

  getByMFID(mfid) {
    const id = this.getID(mfid);

    if (id != null) {
      return this._apps[id];
    }
  }

  getID(mfid) {
    return this._byMFID[_dataTypes.MFID.canonical(mfid)];
  }

  getOwnByID(id) {
    return this._ownApps[id];
  }

  getOwnByMFID(mfid) {
    const id = this.getID(mfid);

    if (id != null) {
      return this._ownApps[id];
    }
  }

  getAnyByID(id) {
    return this.getByID((0, _utilsId.idType)(id)) || this.getOwnByID((0, _utilsId.idType)(id));
  }

  getAppsForUser(id) {
    const apps = [];

    if (this._appsByUser[id]) {
      this._appsByUser[id].forEach(id => {
        const app = this.getByID(id);

        if (app) {
          apps.push(app);
        }
      });
    }

    return apps;
  } // App lifecycle


  add(manifest, userID, permissionsSettings) {
    if ((0, _appManifest.validateManifest)(manifest) === 'valid') {
      throw new Error('Invalid manifest');
    }

    const appID = (0, _utilsId.uniqueID)();
    const app = new _App.default({
      appID,
      manifest,
      installationState: 'pending'
    });
    app.setPermissionsSettings(userID, permissionsSettings);
    this._apps[appID] = app;
    this._byHash[manifest.updateHash] = appID;
    this._byMFID[_dataTypes.MFID.canonical(manifest.id)] = appID;
    return app;
  }

  updateAppDetails(params) {
    const app = this.getOwnByID((0, _utilsId.idType)(params.appID));

    if (app == null) {
      throw new Error('Invalid app');
    }

    app.name = params.name;
    app.contentsPath = params.contentsPath;
    app.editNextVersionNumber(params.version);
  }

  setUserPermissionsSettings(appID, userID, settings) {
    const app = this.getAnyByID(appID);

    if (app == null) {
      throw new Error('Invalid app');
    }

    app.setPermissionsSettings(userID, settings);
  }

  setUserPermission(appID, userID, key, value) {
    const app = this.getAnyByID(appID);

    if (app == null) {
      throw new Error('Invalid app');
    }

    app.setPermission(userID, key, value);
  }

  approveContacts(appID, userID, contactsToApprove) {
    const app = this.getAnyByID(appID);

    if (app == null) {
      throw new Error('Invalid app');
    }

    return app.approveContacts((0, _utilsId.idType)(userID), contactsToApprove);
  }

  removeUser(appID, userID) {
    const app = this.getAnyByID(appID);

    if (app == null) {
      throw new Error('Invalid app');
    }

    app.removeUser(userID);
  }

  remove(id) {
    // TODO: support removing own apps - might be other method/flag to avoid accidents?
    const app = this.getByID(id);

    if (app != null) {
      if (app instanceof _OwnApp.default) {
        delete this._byMFID[app.mfid];
        delete this._ownApps[id];
      } else {
        // TODO: handle "clean" option to remove the app contents
        delete this._byHash[app.manifest.updateHash];
        delete this._byMFID[app.manifest.id];
        delete this._apps[id];
      }
    }
  }

  createSession(appID, userID) {
    const app = this.getAnyByID(appID);

    if (app == null) {
      throw new Error('Invalid app');
    }

    return app.createSession(userID);
  } // Updates


  hasUpdate(id) {
    return this._updates[id] !== null;
  }

  getUpdate(id) {
    return this._updates[id];
  }

  createUpdate(appID, manifest) {
    const app = this.getByID(appID);

    if (app == null) {
      throw new Error('Invalid app');
    }

    const permissionsDiff = (0, _appPermissions.getRequirementsDifference)(app.manifest.permissions, manifest.permissions);
    const hasRequiredPermissionsChanges = Object.keys(permissionsDiff.required.added).length > 0 || Object.keys(permissionsDiff.required.changed).length > 0; // When required permissions are added or changed,
    // reset `permissionsChecked` to false so it requires user action before any grant

    const settings = hasRequiredPermissionsChanges ? Object.keys(app.settings).reduce((acc, userID) => {
      acc[userID] = _objectSpread({}, app.settings[(0, _utilsId.idType)(userID)], {
        permissionsChecked: false
      });
      return acc;
    }, {}) : _objectSpread({}, app.settings);
    const update = {
      app: new _App.default({
        appID,
        manifest,
        installationState: 'pending',
        settings
      }),
      hasRequiredPermissionsChanges
    };
    this._updates[appID] = update;
    return update;
  } // Own apps


  create(params) {
    const app = _OwnApp.default.create(params);

    this._byMFID[_dataTypes.MFID.canonical(app.mfid)] = app.id;
    this._ownApps[app.id] = app;
    return app;
  }

  createNextVersion(id, version, permissions) {
    const app = this.getOwnByID(id);

    if (app == null) {
      throw new Error('App not found');
    }

    app.createNextVersion(version, permissions);
  }

}

exports.default = AppsRepository;

_defineProperty(AppsRepository, "fromJSON", serialized => {
  return new AppsRepository({
    // $FlowFixMe: mapping type
    apps: appsFromJSON(serialized.apps),
    // $FlowFixMe: mapping type
    updates: updatesFromJSON(serialized.updates),
    // $FlowFixMe: mapping type
    ownApps: ownAppsFromJSON(serialized.ownApps)
  });
});

_defineProperty(AppsRepository, "toJSON", registry => ({
  // $FlowFixMe: mapping type
  apps: appsToJSON(registry.apps),
  // $FlowFixMe: mapping type
  updates: updatesToJSON(registry.updates),
  // $FlowFixMe: mapping type
  ownApps: ownAppsToJSON(registry.ownApps)
}));