"use strict";

exports.__esModule = true;
exports.default = void 0;

var _appPermissions = require("@mainframe/app-permissions");

var _utilsId = require("@mainframe/utils-id");

var _feed = require("../swarm/feed");

var _AbstractApp = _interopRequireDefault(require("./AbstractApp"));

var _Session = _interopRequireDefault(require("./Session"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class OwnApp extends _AbstractApp.default {
  // $FlowFixMe: extending params
  // $FlowFixMe: extending App
  constructor(params) {
    super(params);

    _defineProperty(this, "_data", void 0);

    _defineProperty(this, "_updateFeed", void 0);

    _defineProperty(this, "_versions", void 0);

    this._data = params.data;
    this._updateFeed = params.updateFeed;
    this._versions = params.versions;
  } // Getters


  get data() {
    return this._data;
  }

  get contentsPath() {
    return this._data.contentsPath;
  }

  get mfid() {
    return this._data.mfid;
  }

  get updateFeed() {
    return this._updateFeed;
  }

  get updateFeedHash() {
    return this._updateFeed.feedHash;
  }

  get versions() {
    return this._versions;
  }

  getVersionData(version) {
    return this._versions[version || this._data.version];
  } // Setters


  set name(name) {
    this._data.name = name;
  }

  set contentsPath(path) {
    this._data.contentsPath = path;
  }

  editNextVersionNumber(version) {
    if (version === this._data.version) {
      return;
    }

    if (this._versions[version]) {
      throw new Error('Version already exists');
    }

    const latestVersion = _objectSpread({}, this._versions[this._data.version]); // $FlowFixMe Spread


    latestVersion.version = version;
    delete this._versions[this._data.version];
    this._versions[version] = latestVersion;
    this._data.version = version;
  }

  createNextVersion(version, permissions) {
    if (this._versions[version] != null) {
      throw new Error('Version already exists');
    }

    const latestVersion = this._versions[this._data.version];
    this._versions[version] = {
      permissions: permissions || latestVersion.permissions
    };
    this._data.version = version;
  }

  setContentsHash(hash, version) {
    const v = version || this._data.version;
    const versionData = this._versions[v];

    if (versionData == null) {
      throw new Error('Invalid version');
    }

    if (versionData.versionHash != null) {
      throw new Error('Manifest has already been published');
    }

    versionData.contentsHash = hash;
    this._versions[v] = versionData;
  }

  setVersionHash(hash, version) {
    const v = version || this._data.version;
    const versionData = this._versions[v];

    if (versionData == null) {
      throw new Error('Invalid version');
    }

    if (versionData.contentsHash == null) {
      throw new Error('Contents have not yet been published');
    }

    versionData.versionHash = hash;
    this._versions[v] = versionData;
  }

  setPermissionsRequirements(permissions, version) {
    const v = version || this._data.version;
    const versionData = this._versions[v];

    if (versionData == null) {
      throw new Error('Invalid version');
    }

    const requirements = (0, _appPermissions.createRequirements)(permissions.required, permissions.optional);
    versionData.permissions = requirements;
    this._versions[v] = versionData;
  } // Session


  createSession(userID) {
    const versionData = this._versions[this._data.version];

    if (versionData == null) {
      throw new Error('Invalid version');
    } // TODO: Make permissions experience similar to installed
    //       apps to allow developers to fully test flows


    const appPermissions = _objectSpread({}, versionData.permissions.required, {
      WEB_REQUEST: (0, _appPermissions.createWebRequestGrant)(versionData.permissions.required.WEB_REQUEST)
    });

    const userPermissions = this.getPermissions(userID) || {
      WEB_REQUEST: (0, _appPermissions.createWebRequestGrant)()
    };
    const permissions = (0, _appPermissions.mergeGrantsToDetails)(appPermissions, userPermissions);
    const session = new _Session.default(this._appID, userID, permissions.session);
    return {
      sessID: (0, _utilsId.uniqueID)(),
      isDev: true,
      session,
      permissions
    };
  }

}

exports.default = OwnApp;

_defineProperty(OwnApp, "create", params => {
  return new OwnApp({
    appID: (0, _utilsId.uniqueID)(),
    data: {
      contentsPath: params.contentsPath,
      developerID: params.developerID,
      mfid: params.mfid,
      name: params.name || 'Untitled',
      version: params.version
    },
    updateFeed: _feed.OwnFeed.create(),
    versions: {
      [params.version]: {
        permissions: params.permissions || (0, _appPermissions.createRequirements)()
      }
    }
  });
});

_defineProperty(OwnApp, "fromJSON", params => {
  const {
    updateFeed
  } = params,
        others = _objectWithoutProperties(params, ["updateFeed"]);

  return new OwnApp(_objectSpread({}, others, {
    updateFeed: _feed.OwnFeed.fromJSON(updateFeed)
  }));
});

_defineProperty(OwnApp, "toJSON", app => ({
  appID: app._appID,
  data: app._data,
  settings: app._settings,
  updateFeed: _feed.OwnFeed.toJSON(app.updateFeed),
  versions: app._versions
}));