"use strict";

exports.__esModule = true;
exports.default = void 0;

var _utils = require("../utils");

var _SharedAppData = _interopRequireDefault(require("./SharedAppData"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const fromSharedAppData = (0, _utils.mapObject)((0, _utils.mapObject)(_SharedAppData.default.toJSON));
const toSharedAppData = (0, _utils.mapObject)((0, _utils.mapObject)(_SharedAppData.default.fromJSON));

class ContactAppData {
  constructor(params) {
    _defineProperty(this, "_appsByContact", void 0);

    _defineProperty(this, "_contactsByApp", void 0);

    this._appsByContact = params ? params.appsByContact : {}; // Generate reverse index

    this._contactsByApp = {};
    Object.keys(this._appsByContact).forEach(contactID => {
      Object.keys(this._appsByContact[contactID]).forEach(appID => {
        if (this._contactsByApp[appID] == null) {
          this._contactsByApp[appID] = {};
        }

        this._contactsByApp[appID][contactID] = this._appsByContact[contactID][appID];
      });
    });
  }

  get appsByContact() {
    return this._appsByContact;
  }

  get contactsByApp() {
    return this._contactsByApp;
  }

  getSharedData(appID, contactID) {
    return this._appsByContact[contactID] && this._appsByContact[contactID][appID];
  }

  createSharedData(appID, contactID, optional) {
    // $FlowFixMe: optional mismatch
    const shared = _SharedAppData.default.create(optional);

    if (this._appsByContact[contactID] == null) {
      this._appsByContact[contactID] = {};
    }

    this._appsByContact[contactID][appID] = shared;

    if (this._contactsByApp[appID] == null) {
      this._contactsByApp[appID] = {};
    }

    this._contactsByApp[appID][contactID] = shared;
    return shared;
  }

  deleteContact(contactID) {
    Object.keys(this._appsByContact[contactID]).forEach(appID => delete this._contactsByApp[appID][contactID]);
    delete this._appsByContact[contactID];
  }

  deleteApp(appID) {
    Object.keys(this._contactsByApp[appID]).forEach(contactID => delete this._appsByContact[contactID][appID]);
    delete this._contactsByApp[appID];
  }

}

exports.default = ContactAppData;

_defineProperty(ContactAppData, "fromJSON", params => {
  return new ContactAppData({
    // $FlowFixMe: mapping type
    appsByContact: toSharedAppData(params.appsByContact)
  });
});

_defineProperty(ContactAppData, "toJSON", contactAppData => {
  return {
    // $FlowFixMe: mapping type
    appsByContact: fromSharedAppData(contactAppData.appsByContact)
  };
});