"use strict";

exports.__esModule = true;
exports.default = void 0;

var _apiBzzNode = _interopRequireDefault(require("@erebos/api-bzz-node"));

var _feed = require("../swarm/feed");

var _utils = require("../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const fromAppFeeds = (0, _utils.mapObject)(f => ({
  type: f.type,
  feed: _feed.OwnFeed.toJSON(f.feed)
}));
const toAppFeeds = (0, _utils.mapObject)(f => ({
  type: f.type,
  feed: _feed.OwnFeed.fromJSON(f.feed)
}));
const toAppFeedsPayload = (0, _utils.mapObject)(f => ({
  type: f.type,
  feedHash: f.feed.feedHash
}));

class SharedAppData {
  constructor(sharedFeed, appFeeds) {
    _defineProperty(this, "_sharedFeed", void 0);

    _defineProperty(this, "_appFeeds", void 0);

    this._sharedFeed = sharedFeed;
    this._appFeeds = appFeeds;
  }

  get sharedFeed() {
    return this._sharedFeed;
  }

  get feedHash() {
    return this.sharedFeed.localFeed.feedHash;
  }

  get remoteFeed() {
    return this.sharedFeed.remoteFeed;
  }

  set remoteFeed(remoteFeed) {
    this.sharedFeed.remoteFeed = remoteFeed;
  }

  getAppFeed(key) {
    return this._appFeeds[key];
  }

  createAppFeed(type, key) {
    if (this._appFeeds[key]) throw new Error('App feed key already exists');

    switch (type) {
      case 'json':
        {
          const feed = {
            type,
            feed: _feed.OwnFeed.create(undefined, (0, _feed.randomTopic)())
          };
          this._appFeeds[key] = feed;
          return feed;
        }

      default:
        throw new Error(`Unexpected feed type ${type}`);
    }
  }

  async syncManifest(bzz) {
    return this.sharedFeed.syncManifest(bzz);
  }

  async publish(bzz) {
    return this.sharedFeed.publish(bzz, this._generateFeedPayload());
  }

  _generateFeedPayload() {
    return {
      version: '1.0.0',
      // $FlowFixMe: mapping type
      feeds: toAppFeedsPayload(this._appFeeds)
    };
  }

}

exports.default = SharedAppData;

_defineProperty(SharedAppData, "create", optional => {
  return new SharedAppData(_feed.BidirectionalFeed.create(optional), {});
});

_defineProperty(SharedAppData, "fromJSON", serialized => new SharedAppData(_feed.BidirectionalFeed.fromJSON(serialized.sharedFeed), // $FlowFixMe: mapping type
toAppFeeds(serialized.appFeeds)));

_defineProperty(SharedAppData, "toJSON", sharedAppData => ({
  sharedFeed: _feed.BidirectionalFeed.toJSON(sharedAppData.sharedFeed),
  // $FlowFixMe: mapping type
  appFeeds: fromAppFeeds(sharedAppData._appFeeds)
}));