"use strict";

exports.__esModule = true;
exports.default = void 0;

var _rxjs = require("rxjs");

var _ContextEvents = _interopRequireDefault(require("./ContextEvents"));

var _ContextIO = _interopRequireDefault(require("./ContextIO"));

var _ContextMutations = _interopRequireDefault(require("./ContextMutations"));

var _ContextQueries = _interopRequireDefault(require("./ContextQueries"));

var _ContextSubscriptions = _interopRequireDefault(require("./ContextSubscriptions"));

var _FeedsHandler = require("./FeedsHandler");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class ClientContext extends _rxjs.Subject {
  constructor(params) {
    super();

    _defineProperty(this, "env", void 0);

    _defineProperty(this, "events", void 0);

    _defineProperty(this, "io", void 0);

    _defineProperty(this, "log", void 0);

    _defineProperty(this, "mutations", void 0);

    _defineProperty(this, "notify", void 0);

    _defineProperty(this, "queries", void 0);

    _defineProperty(this, "socket", void 0);

    _defineProperty(this, "subscriptions", void 0);

    _defineProperty(this, "vaults", void 0);

    _defineProperty(this, "contactsFeeds", void 0);

    this.env = params.env;
    this.log = params.log;
    this.notify = params.notify;
    this.socket = params.socket;
    this.vaults = params.vaults;
    this.events = new _ContextEvents.default(this);
    this.io = new _ContextIO.default(this);
    this.mutations = new _ContextMutations.default(this);
    this.queries = new _ContextQueries.default(this);
    this.subscriptions = new _ContextSubscriptions.default(this);
    this.contactsFeeds = new _FeedsHandler.ContactsFeedsHandler(this);
  }

  get openVault() {
    const vault = this.vaults.getVault(this.socket);

    if (vault == null) {
      throw new Error('Vault is not open');
    }

    return vault;
  }

  async clear() {
    this.unsubscribe();
    this.events.clear();
    this.io.clear();
    await this.subscriptions.clear();
    const openVault = this.vaults.getVault(this.socket);

    if (openVault != null) {
      await openVault.save();
    }
  }

}

exports.default = ClientContext;