"use strict";

exports.__esModule = true;
exports.default = void 0;

var _apiBzzNode = _interopRequireDefault(require("@erebos/api-bzz-node"));

var _apiPss = _interopRequireDefault(require("@erebos/api-pss"));

var _secp256k = require("@erebos/secp256k1");

var _rpcWsNode = _interopRequireDefault(require("@mainframe/rpc-ws-node"));

var _eth = require("@mainframe/eth");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class ContextIO {
  constructor(context) {
    _defineProperty(this, "_context", void 0);

    _defineProperty(this, "_rpc", void 0);

    _defineProperty(this, "_bzz", void 0);

    _defineProperty(this, "_pss", void 0);

    _defineProperty(this, "_ethClient", void 0);

    this._context = context;
  }

  get rpc() {
    if (this._rpc == null) {
      this._rpc = (0, _rpcWsNode.default)(this._context.openVault.settings.pssURL);
    }

    return this._rpc;
  }

  get bzz() {
    if (this._bzz == null) {
      this._bzz = new _apiBzzNode.default({
        url: this._context.openVault.settings.bzzURL,
        signFeedDigest: async (digest, key) => {
          if (key == null) {
            throw new Error('Missing signing key');
          }

          return (0, _secp256k.sign)(digest, key);
        }
      });
    }

    return this._bzz;
  }

  get pss() {
    if (this._pss == null) {
      this._pss = new _apiPss.default(this.rpc);
    }

    return this._pss;
  }

  get eth() {
    if (this._ethClient == null) {
      this._ethClient = new _eth.EthClient(this._context.openVault.settings.ethURL);
    }

    return this._ethClient;
  }

  clear() {
    this._bzz = undefined;
    this._pss = undefined;
    this._ethClient = undefined;

    if (this._rpc != null) {
      this._rpc.disconnect();
    }
  }

}

exports.default = ContextIO;