"use strict";

exports.__esModule = true;
exports.default = void 0;

var _utilsId = require("@mainframe/utils-id");

var _feed = require("../swarm/feed");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class ContextQueries {
  constructor(context) {
    _defineProperty(this, "_context", void 0);

    this._context = context;
  } // Apps


  getAppManifestData(appID, version) {
    const {
      openVault
    } = this._context;
    const app = openVault.apps.getOwnByID(appID);

    if (app == null) {
      throw new Error('App not found');
    }

    const devIdentity = openVault.identities.getOwnDeveloper(app.data.developerID);

    if (devIdentity == null) {
      throw new Error('Developer identity not found');
    }

    const versionData = app.getVersionData(version);

    if (versionData == null) {
      throw new Error('Invalid app version');
    }

    return {
      id: app.mfid,
      author: {
        id: devIdentity.id,
        name: devIdentity.profile.name
      },
      name: app.data.name,
      version: app.data.version,
      contentsHash: versionData.contentsHash,
      updateHash: app.updateFeed.feedHash,
      permissions: versionData.permissions
    };
  } // Contacts


  getUserContacts(userID) {
    const {
      identities
    } = this._context.openVault;
    const result = [];
    const contacts = identities.getContactsForUser(userID);

    if (contacts) {
      Object.keys(contacts).forEach(id => {
        const contact = contacts[id];
        const peer = identities.getPeerUser((0, _utilsId.idType)(contact.peerID));

        if (peer) {
          const contactProfile = identities.getContactProfile(contact.localID);

          const profile = _objectSpread({}, peer.profile, contactProfile);

          const contactRes = {
            profile,
            localID: id,
            peerID: contact.peerID,
            publicFeed: peer.publicFeed,
            connectionState: contact.connectionState
          };
          result.push(contactRes);
        }
      });
    }

    return result;
  }

  getAppApprovedContacts(appID, userID) {
    const {
      apps
    } = this._context.openVault;
    const app = apps.getAnyByID(appID);

    if (!app) {
      throw new Error('App not found');
    }

    const contactIDs = Object.keys(app.getApprovedContacts(userID));
    return this.getAppUserContacts(appID, userID, contactIDs);
  }

  getAppUserContacts(appID, userID, contactIDs) {
    const {
      apps,
      identities
    } = this._context.openVault;
    const app = apps.getAnyByID(appID);

    if (!app) {
      throw new Error('App not found');
    }

    const approvedContacts = app.getApprovedContacts(userID);
    const contacts = this.getUserContacts(userID);
    return contactIDs.map(id => {
      const approvedContact = approvedContacts[id];
      const contactData = {
        id,
        data: null
      };

      if (approvedContact) {
        const contact = contacts.find(c => c.localID === approvedContact.localID);

        if (contact) {
          contactData.data = {
            profile: identities.getContactProfile(contact.localID)
          };
        }
      }

      return contactData;
    });
  }

  async getContactAppFeeds(appID, contactID) {
    const {
      openVault,
      io
    } = this._context;
    const app = openVault.apps.getByID(appID);

    if (!app) {
      throw new Error('App not found');
    }

    const sharedAppID = app.updateFeedHash;

    if (!sharedAppID) {
      throw new Error('updateFeedHash missing for app');
    }

    const sharedData = openVault.contactAppData.getSharedData(sharedAppID, contactID);

    if (!sharedData || !sharedData.remoteFeed) {
      return {};
    }

    const remoteData = await (0, _feed.fetchJSON)(io.bzz, sharedData.remoteFeed); // TODO: validate payload version

    return remoteData.feeds || {};
  }

  getContactLocalIDByAppApprovedID(appID, userID, contactID) {
    const app = this._context.openVault.apps.getByID(appID);

    if (!app) {
      throw new Error('App not found');
    }

    return app.getLocalIDByApprovedID(userID, contactID);
  } // Wallets


  getUserEthWallets(id) {
    const {
      identityWallets,
      wallets
    } = this._context.openVault;

    if (identityWallets.walletsByIdentity[id]) {
      return Object.keys(identityWallets.walletsByIdentity[id]).reduce((acc, wid) => {
        const wallet = wallets.getEthWalletByID(wid);

        if (wallet) {
          acc.push({
            localID: wallet.localID,
            name: wallet.name,
            type: wallet.type,
            accounts: wallet.getAccounts()
          });
        }

        return acc;
      }, []);
    }

    return [];
  }

  getUserEthWalletForAccount(userID, address) {
    const wallets = this.getUserEthWallets(userID);
    return wallets.find(w => w.accounts.includes(address));
  }

  getUserEthAccounts(userID) {
    return this.getUserEthWallets(userID).reduce((acc, w) => {
      // $FlowFixMe concat types
      return acc.concat(w.accounts);
    }, []);
  }

  getUserDefaultEthAccount(userID) {
    const {
      identityWallets
    } = this._context.openVault;
    const defaultAcc = identityWallets.getDefaultEthWallet(userID);

    if (defaultAcc) {
      return defaultAcc;
    } // Use first account if no default


    const wallets = this.getUserEthWallets(userID);

    if (wallets.length && wallets[0].accounts.length) {
      const addr = wallets[0].accounts[0];

      this._context.mutations.setUsersDefaultWallet(userID, addr);

      return addr;
    }
  }

}

exports.default = ContextQueries;