"use strict";

exports.__esModule = true;
exports.default = exports.ContextSubscription = void 0;

var _utilsId = require("@mainframe/utils-id");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class ContextSubscription {
  constructor(method, data) {
    _defineProperty(this, "_id", void 0);

    _defineProperty(this, "_method", void 0);

    _defineProperty(this, "data", void 0);

    this._id = (0, _utilsId.uniqueID)();
    this._method = method;
    this.data = data;
  }

  get id() {
    return this._id;
  }

  get method() {
    return this._method;
  }

  async dispose() {}

}

exports.ContextSubscription = ContextSubscription;

class ContextSubscriptions {
  constructor(context) {
    _defineProperty(this, "_context", void 0);

    _defineProperty(this, "_subscriptions", {});

    this._context = context;
  }

  set(sub) {
    this._subscriptions[sub.id] = sub;
  }

  create(method, data) {
    const sub = new ContextSubscription(method, data);
    this.set(sub);
    return sub;
  }

  async remove(id) {
    const sub = this._subscriptions[id];

    if (sub != null) {
      await sub.dispose();
      delete this._subscriptions[id];
    }
  }

  notify(id, result) {
    const sub = this._subscriptions[id];

    if (sub != null) {
      this._context.notify(sub.method, {
        subscription: id,
        result
      });
    }
  }

  async clear() {
    const disposeSubs = Object.values(this._subscriptions).map(sub => {
      // $FlowFixMe: Object.values() losing type
      return sub.dispose();
    });
    await Promise.all(disposeSubs);
    this._subscriptions = {};
  }

}

exports.default = ContextSubscriptions;