"use strict";

exports.__esModule = true;
exports.default = exports.updateAppDetailsMutation = void 0;

var _graphql = require("graphql");

var _graphqlRelay = require("graphql-relay");

var _objects = require("./objects");

const viewerOutput = {
  type: new _graphql.GraphQLNonNull(_objects.viewer),
  resolve: () => ({})
};
const supportedWalletsEnum = new _graphql.GraphQLEnumType({
  name: 'SupportedWallets',
  values: {
    ETHEREUM: {
      value: 'ethereum'
    }
  }
});
const deleteWalletMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'DeleteWallet',
  inputFields: {
    walletID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    type: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    await ctx.mutations.deleteWallet('ethereum', args.type, args.walletID);
    return {};
  }
});
const addHDWalletAccountMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'AddHDWalletAccount',
  inputFields: {
    walletID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    index: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLInt)
    },
    userID: {
      type: _graphql.GraphQLString
    }
  },
  outputFields: {
    address: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: payload => payload.address
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    const address = await ctx.mutations.addHDWalletAccount(args);
    return {
      address
    };
  }
});
const importHDWalletMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'ImportHDWallet',
  inputFields: {
    blockchain: {
      type: new _graphql.GraphQLNonNull(supportedWalletsEnum)
    },
    mnemonic: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    userID: {
      type: _graphql.GraphQLString
    }
  },
  outputFields: {
    hdWallet: {
      type: _objects.ethHdWallet,
      resolve: payload => payload
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    const wallet = await ctx.mutations.importHDWallet({
      blockchain: args.blockchain,
      mnemonic: args.mnemonic,
      name: args.name,
      userID: args.userID
    });
    return {
      localID: wallet.localID,
      accounts: wallet.getAccounts()
    };
  }
});
const createHDWalletMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'CreateHDWallet',
  inputFields: {
    blockchain: {
      type: new _graphql.GraphQLNonNull(supportedWalletsEnum)
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    userID: {
      type: _graphql.GraphQLString
    }
  },
  outputFields: {
    hdWallet: {
      type: _objects.ethHdWallet,
      resolve: payload => payload
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    const wallet = await ctx.mutations.createHDWallet(args.blockchain, args.name, args.userID);
    return {
      localID: wallet.localID,
      mnemonic: wallet.mnemonic,
      accounts: wallet.getAccounts()
    };
  }
});
const addLedgerWalletAccountsMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'AddLedgerWalletAccounts',
  inputFields: {
    indexes: {
      type: new _graphql.GraphQLList(_graphql.GraphQLInt)
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    userID: {
      type: _graphql.GraphQLString
    }
  },
  outputFields: {
    addresses: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString),
      resolve: payload => payload.addresses
    },
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: payload => payload.localID
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    const res = await ctx.mutations.addLedgerWalletAccounts(args.indexes, args.name, args.userID);
    return res;
  }
});
const setDefaultWalletMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'SetDefaultWallet',
  inputFields: {
    userID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    address: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    await ctx.mutations.setUsersDefaultWallet(args.userID, args.address);
    return {};
  }
});
const setEthNetworkMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'SetEthNetwork',
  inputFields: {
    url: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    await ctx.mutations.setEthNetwork(args.url);
    return {};
  }
});
const userProfileInput = new _graphql.GraphQLInputObjectType({
  name: 'UserProfileInput',
  fields: () => ({
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    avatar: {
      type: _graphql.GraphQLString
    }
  })
});
const createUserIdentityMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'CreateUserIdentity',
  inputFields: {
    profile: {
      type: new _graphql.GraphQLNonNull(userProfileInput)
    },
    private: {
      type: _graphql.GraphQLBoolean
    }
  },
  outputFields: {
    user: {
      type: _objects.ownUserIdentity,
      resolve: payload => payload.user
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    const user = await ctx.mutations.createUser(args.profile, args.private);
    return {
      user
    };
  }
});
const createDeveloperIdentityMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'CreateDeveloperIdentity',
  inputFields: {
    profile: {
      type: new _graphql.GraphQLNonNull(userProfileInput)
    }
  },
  outputFields: {
    user: {
      type: _objects.ownDeveloperIdentity,
      resolve: payload => payload.user
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    const user = ctx.openVault.identities.createOwnDeveloper(args.profile);
    await ctx.openVault.save();
    return {
      user
    };
  }
});
const updateProfileInput = new _graphql.GraphQLInputObjectType({
  name: 'UpdateUserProfileInput',
  fields: () => ({
    name: {
      type: _graphql.GraphQLString
    },
    avatar: {
      type: _graphql.GraphQLString
    },
    ethAddress: {
      type: _graphql.GraphQLString
    }
  })
});
const updateProfileMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'UpdateProfile',
  inputFields: {
    userID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    profile: {
      type: new _graphql.GraphQLNonNull(updateProfileInput)
    },
    privateProfile: {
      type: _graphql.GraphQLBoolean
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    await ctx.mutations.updateUser(args.userID, args.profile, args.privateProfile);
    return {};
  }
});
const setUserProfileVisibilityMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'SetUserProfileVisibility',
  inputFields: {
    userID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    visibile: {
      type: _graphql.GraphQLBoolean
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    await ctx.mutations.setUserProfileVisibility(args.userID, args.visibile);
    return {};
  }
});
const addContactMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'AddContact',
  inputFields: {
    userID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    publicFeed: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    aliasName: {
      type: _graphql.GraphQLString
    }
  },
  outputFields: {
    contact: {
      type: _objects.contact,
      resolve: payload => payload.contact
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    return await ctx.mutations.createContactFromFeed(args.userID, args.publicFeed, args.aliasName);
  }
});
const deleteContactMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'DeletContact',
  inputFields: {
    contactID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    ctx.openVault.identities.deleteContact(args.userID, args.contactID);
    await ctx.openVault.save();
  }
});
const appPermissionDefinitionsInput = new _graphql.GraphQLInputObjectType({
  name: 'AppPermissionDefinitionsInput',
  fields: () => ({
    BLOCKCHAIN_SEND: {
      type: _graphql.GraphQLBoolean
    },
    COMMS_CONTACT: {
      type: _graphql.GraphQLBoolean
    },
    CONTACTS_READ: {
      type: _graphql.GraphQLBoolean
    },
    WEB_REQUEST: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString)
    }
  })
});
const appPermissionsRequirementsInput = new _graphql.GraphQLInputObjectType({
  name: 'AppPermissionsRequirementsInput',
  fields: () => ({
    optional: {
      type: new _graphql.GraphQLNonNull(appPermissionDefinitionsInput)
    },
    required: {
      type: new _graphql.GraphQLNonNull(appPermissionDefinitionsInput)
    }
  })
});
const appCreateMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'AppCreateMutation',
  inputFields: {
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    contentsPath: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    version: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    developerID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    permissionsRequirements: {
      type: new _graphql.GraphQLNonNull(appPermissionsRequirementsInput)
    }
  },
  outputFields: {
    app: {
      type: _objects.ownApp,
      resolve: payload => payload.app
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    const app = await ctx.mutations.createApp(args);
    return {
      app
    };
  }
});
const setAppPermissionsRequirementsMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'SetAppPermissionsRequirements',
  inputFields: {
    appID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    permissionsRequirements: {
      type: new _graphql.GraphQLNonNull(appPermissionsRequirementsInput)
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (args, ctx) => {
    await ctx.mutations.setAppPermissionsRequirements(args.appID, args.permissionsRequirements);
    return {};
  }
});
const manifestAuthorInput = new _graphql.GraphQLInputObjectType({
  name: 'ManifestAuthorInput',
  fields: () => ({
    id: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  })
});
const appManifestInput = new _graphql.GraphQLInputObjectType({
  name: 'AppManifestInput',
  fields: () => ({
    id: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    version: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    contentsHash: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    updateHash: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    permissions: {
      type: new _graphql.GraphQLNonNull(appPermissionsRequirementsInput)
    },
    author: {
      type: new _graphql.GraphQLNonNull(manifestAuthorInput)
    }
  })
});
const webRequestGrantInput = new _graphql.GraphQLInputObjectType({
  name: 'WebRequestGrantInput',
  fields: () => ({
    granted: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString)
    },
    denied: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString)
    }
  })
});
const permissionGrantsInput = new _graphql.GraphQLInputObjectType({
  name: 'PermissionGrantsInput',
  fields: () => ({
    BLOCKCHAIN_SEND: {
      type: _graphql.GraphQLBoolean
    },
    COMMS_CONTACT: {
      type: _graphql.GraphQLBoolean
    },
    CONTACTS_READ: {
      type: _graphql.GraphQLBoolean
    },
    WEB_REQUEST: {
      type: new _graphql.GraphQLNonNull(webRequestGrantInput)
    }
  })
});
const appPermissionSettingsInput = new _graphql.GraphQLInputObjectType({
  name: 'AppPermissionsSettingsInput',
  fields: () => ({
    permissionsChecked: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLBoolean)
    },
    grants: {
      type: new _graphql.GraphQLNonNull(permissionGrantsInput)
    }
  })
});
const appInstallMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'AppInstallMutation',
  inputFields: {
    userID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    manifest: {
      type: new _graphql.GraphQLNonNull(appManifestInput)
    },
    permissionsSettings: {
      type: new _graphql.GraphQLNonNull(appPermissionSettingsInput)
    }
  },
  outputFields: {
    app: {
      type: _objects.app,
      resolve: payload => payload.app
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async ({
    userID,
    manifest,
    permissionsSettings
  }, ctx) => {
    const app = await ctx.mutations.installApp({
      userID,
      manifest,
      permissionsSettings
    });
    return {
      app
    };
  }
});
const updateAppDetailsMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'UpdateAppDetails',
  inputFields: {
    appID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    contentsPath: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    version: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  },
  outputFields: {
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (params, ctx) => {
    await ctx.mutations.updateAppDetails(params);
    return {};
  }
});
exports.updateAppDetailsMutation = updateAppDetailsMutation;
const publishAppVersionMutation = (0, _graphqlRelay.mutationWithClientMutationId)({
  name: 'PublishAppVersion',
  inputFields: {
    appID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    version: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  },
  outputFields: {
    versionHash: {
      type: (0, _graphql.GraphQLNonNull)(_graphql.GraphQLString)
    },
    viewer: viewerOutput
  },
  mutateAndGetPayload: async (params, ctx) => {
    const versionHash = await ctx.mutations.publishApp(params);
    return {
      versionHash
    };
  }
});

var _default = new _graphql.GraphQLObjectType({
  name: 'Mutation',
  fields: () => ({
    // Apps
    createApp: appCreateMutation,
    installApp: appInstallMutation,
    setAppPermissionsRequirements: setAppPermissionsRequirementsMutation,
    publishAppVersion: publishAppVersionMutation,
    updateAppDetails: updateAppDetailsMutation,
    // Users
    addContact: addContactMutation,
    createUserIdentity: createUserIdentityMutation,
    createDeveloperIdentity: createDeveloperIdentityMutation,
    deleteContact: deleteContactMutation,
    setDefaultWallet: setDefaultWalletMutation,
    setUserProfileVisibility: setUserProfileVisibilityMutation,
    updateProfile: updateProfileMutation,
    // Wallets
    addHDWalletAccount: addHDWalletAccountMutation,
    addLedgerWalletAccounts: addLedgerWalletAccountsMutation,
    createHDWallet: createHDWalletMutation,
    deleteWallet: deleteWalletMutation,
    importHDWallet: importHDWalletMutation,
    setEthNetwork: setEthNetworkMutation
  })
});

exports.default = _default;