"use strict";

exports.__esModule = true;
exports.viewer = exports.settings = exports.wallets = exports.ethWallets = exports.ethHdWallet = exports.ethLedgerWallet = exports.walletAccount = exports.walletBalances = exports.peers = exports.identities = exports.apps = exports.contacts = exports.contact = exports.connectionState = exports.peer = exports.peerUserIdentity = exports.peerDeveloperIdentity = exports.peerAppIdentity = exports.ownUserIdentity = exports.ownDeveloperIdentity = exports.ownAppIdentity = exports.namedProfile = exports.genericProfile = exports.userWallet = exports.ownApp = exports.app = exports.appVersionData = exports.appManifestData = exports.appAuthor = exports.appPermissionsRequirements = exports.appPermissionDefinitions = exports.appUser = exports.appUserSettings = exports.appPermissionSettings = exports.appPermissionGrants = exports.webRequestGrants = exports.idResolver = exports.nodeField = exports.nodeInterface = void 0;

var _utilsId = require("@mainframe/utils-id");

var _graphql = require("graphql");

var _graphqlRelay = require("graphql-relay");

var _app = require("../app");

var _identity = require("../identity");

var _LedgerWallet = _interopRequireDefault(require("../wallet/LedgerWallet"));

var _HDWallet = _interopRequireDefault(require("../wallet/HDWallet"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const {
  nodeInterface,
  nodeField
} = (0, _graphqlRelay.nodeDefinitions)((globalId, ctx) => {
  if (globalId === 'viewer') {
    return {};
  }

  const {
    type,
    id
  } = (0, _graphqlRelay.fromGlobalId)(globalId);
  const typedID = (0, _utilsId.idType)(id);

  switch (type) {
    case 'App':
      return ctx.openVault.apps.getByID(typedID);

    case 'EthHDWallet':
      return ctx.openVault.wallets.getEthHDWallet(typedID);

    case 'EthLedgerWallet':
      return ctx.openVault.wallets.getEthLedgerWallet(typedID);

    case 'OwnApp':
      return ctx.openVault.apps.getOwnByID(typedID);

    case 'OwnAppIdentity':
      return ctx.openVault.identities.getOwnApp(typedID);

    case 'OwnDeveloperIdentity':
      return ctx.openVault.identities.getOwnDeveloper(typedID);

    case 'OwnUserIdentity':
      return ctx.openVault.identities.getOwnUser(typedID);

    case 'PeerAppIdentity':
      return ctx.openVault.identities.getPeerApp(typedID);

    case 'PeerDeveloperIdentity':
      return ctx.openVault.identities.getPeerDeveloper(typedID);

    case 'PeerUserIdentity':
      return ctx.openVault.identities.getPeerUser(typedID);

    default:
      return null;
  }
}, obj => {
  if (obj instanceof _app.App) {
    return app;
  }

  if (obj instanceof _HDWallet.default) {
    return ethHdWallet;
  }

  if (obj instanceof _LedgerWallet.default) {
    return ethLedgerWallet;
  }

  if (obj instanceof _app.OwnApp) {
    return ownApp;
  }

  if (obj instanceof _identity.OwnAppIdentity) {
    return ownAppIdentity;
  }

  if (obj instanceof _identity.OwnDeveloperIdentity) {
    return ownDeveloperIdentity;
  }

  if (obj instanceof _identity.OwnUserIdentity) {
    return ownUserIdentity;
  }

  if (obj instanceof _identity.PeerAppIdentity) {
    return peerAppIdentity;
  }

  if (obj instanceof _identity.PeerDeveloperIdentity) {
    return peerDeveloperIdentity;
  }

  if (obj instanceof _identity.PeerUserIdentity) {
    return peerUserIdentity;
  }

  return null;
});
exports.nodeField = nodeField;
exports.nodeInterface = nodeInterface;
const idResolver = (0, _graphqlRelay.globalIdField)(null, obj => obj.localID);
exports.idResolver = idResolver;
const webRequestGrants = new _graphql.GraphQLObjectType({
  name: 'WebRequestGrants',
  fields: () => ({
    granted: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString)
    },
    denied: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString)
    }
  })
});
exports.webRequestGrants = webRequestGrants;
const appPermissionGrants = new _graphql.GraphQLObjectType({
  name: 'AppPermissions',
  fields: () => ({
    BLOCKCHAIN_SEND: {
      type: _graphql.GraphQLBoolean
    },
    CONTACTS_READ: {
      type: _graphql.GraphQLBoolean
    },
    WEB_REQUEST: {
      type: new _graphql.GraphQLNonNull(webRequestGrants)
    }
  })
});
exports.appPermissionGrants = appPermissionGrants;
const appPermissionSettings = new _graphql.GraphQLObjectType({
  name: 'AppPermissionsSettings',
  fields: () => ({
    permissionsChecked: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLBoolean)
    },
    grants: {
      type: new _graphql.GraphQLNonNull(appPermissionGrants)
    }
  })
});
exports.appPermissionSettings = appPermissionSettings;
const appUserSettings = new _graphql.GraphQLObjectType({
  name: 'AppUserSettings',
  fields: () => ({
    permissionsSettings: {
      type: new _graphql.GraphQLNonNull(appPermissionSettings)
    }
  })
});
exports.appUserSettings = appUserSettings;
const appUser = new _graphql.GraphQLObjectType({
  name: 'AppUser',
  fields: () => ({
    id: idResolver,
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    identity: {
      type: new _graphql.GraphQLNonNull(ownUserIdentity),
      resolve: (self, args, ctx) => {
        return ctx.openVault.identities.getOwnUser(self.localID);
      }
    },
    settings: {
      type: new _graphql.GraphQLNonNull(appUserSettings)
    }
  })
});
exports.appUser = appUser;
const appPermissionDefinitions = new _graphql.GraphQLObjectType({
  name: 'AppPermissionDefinitions',
  fields: () => ({
    WEB_REQUEST: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString)
    },
    BLOCKCHAIN_SEND: {
      type: _graphql.GraphQLBoolean
    },
    COMMS_CONTACT: {
      type: _graphql.GraphQLBoolean
    },
    CONTACTS_READ: {
      type: _graphql.GraphQLBoolean
    }
  })
});
exports.appPermissionDefinitions = appPermissionDefinitions;
const appPermissionsRequirements = new _graphql.GraphQLObjectType({
  name: 'AppPermissionsRequirements',
  fields: () => ({
    optional: {
      type: new _graphql.GraphQLNonNull(appPermissionDefinitions)
    },
    required: {
      type: new _graphql.GraphQLNonNull(appPermissionDefinitions)
    }
  })
});
exports.appPermissionsRequirements = appPermissionsRequirements;
const appAuthor = new _graphql.GraphQLObjectType({
  name: 'AppAuthor',
  fields: () => ({
    id: {
      type: _graphql.GraphQLString
    },
    name: {
      type: _graphql.GraphQLString
    }
  })
});
exports.appAuthor = appAuthor;
const appManifestData = new _graphql.GraphQLObjectType({
  name: 'AppManifestData',
  fields: () => ({
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    version: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    permissions: {
      type: new _graphql.GraphQLNonNull(appPermissionsRequirements)
    },
    author: {
      type: new _graphql.GraphQLNonNull(appAuthor)
    }
  })
});
exports.appManifestData = appManifestData;
const appVersionData = new _graphql.GraphQLObjectType({
  name: 'AppVersionData',
  fields: () => ({
    version: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    versionHash: {
      type: _graphql.GraphQLString
    },
    permissions: {
      type: new _graphql.GraphQLNonNull(appPermissionsRequirements)
    }
  })
});
exports.appVersionData = appVersionData;
const app = new _graphql.GraphQLObjectType({
  name: 'App',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: (0, _graphqlRelay.globalIdField)(),
    mfid: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      resolve: self => self.id
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: self => self.manifest.name
    },
    manifest: {
      type: new _graphql.GraphQLNonNull(appManifestData)
    },
    users: {
      type: new _graphql.GraphQLList(appUser),
      resolve: ({
        settings
      }) => {
        return Object.keys(settings).map(id => ({
          localID: id,
          settings: settings[id]
        }));
      }
    }
  })
});
exports.app = app;
const ownApp = new _graphql.GraphQLObjectType({
  name: 'OwnApp',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: (0, _graphqlRelay.globalIdField)(),
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      resolve: self => self.id
    },
    mfid: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    name: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: self => self.data.name
    },
    contentsPath: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: self => self.data.contentsPath
    },
    updateFeedHash: {
      type: _graphql.GraphQLString
    },
    versions: {
      type: new _graphql.GraphQLNonNull(new _graphql.GraphQLList(new _graphql.GraphQLNonNull(appVersionData))),
      resolve: ({
        versions
      }) => {
        return Object.keys(versions).map(version => _objectSpread({
          version: version
        }, versions[version]));
      }
    },
    developer: {
      type: new _graphql.GraphQLNonNull(appAuthor),
      resolve: (self, args, ctx) => {
        const dev = ctx.openVault.identities.getOwnDeveloper(self.data.developerID);
        return {
          id: dev.id,
          name: dev.profile.name
        };
      }
    },
    users: {
      type: new _graphql.GraphQLList(appUser),
      resolve: ({
        settings
      }) => {
        return Object.keys(settings).map(id => ({
          localID: id,
          settings: settings[id]
        }));
      }
    }
  })
});
exports.ownApp = ownApp;
const userWallet = new _graphql.GraphQLObjectType({
  name: 'UserWallet',
  fields: () => ({
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    accounts: {
      type: new _graphql.GraphQLList(_graphql.GraphQLString)
    }
  })
});
exports.userWallet = userWallet;
const genericProfile = new _graphql.GraphQLObjectType({
  name: 'GenericProfile',
  fields: () => ({
    name: {
      type: _graphql.GraphQLString
    },
    avatar: {
      type: _graphql.GraphQLString
    },
    ethAddress: {
      type: _graphql.GraphQLString
    }
  })
});
exports.genericProfile = genericProfile;
const namedProfile = new _graphql.GraphQLObjectType({
  name: 'NamedProfile',
  fields: () => ({
    name: {
      type: (0, _graphql.GraphQLNonNull)(_graphql.GraphQLString)
    },
    avatar: {
      type: _graphql.GraphQLString
    },
    ethAddress: {
      type: _graphql.GraphQLString
    }
  })
});
exports.namedProfile = namedProfile;
const ownAppIdentity = new _graphql.GraphQLObjectType({
  name: 'OwnAppIdentity',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    // TODO: consistent id naming
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    mfid: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      resolve: self => self.id
    },
    pubKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  })
});
exports.ownAppIdentity = ownAppIdentity;
const ownDeveloperIdentity = new _graphql.GraphQLObjectType({
  name: 'OwnDeveloperIdentity',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    mfid: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      resolve: self => self.id
    },
    pubKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    profile: {
      type: new _graphql.GraphQLNonNull(namedProfile)
    }
  })
});
exports.ownDeveloperIdentity = ownDeveloperIdentity;
const ownUserIdentity = new _graphql.GraphQLObjectType({
  name: 'OwnUserIdentity',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    feedHash: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: self => self.publicFeed.feedHash
    },
    mfid: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      resolve: self => self.id
    },
    apps: {
      type: new _graphql.GraphQLList(app),
      resolve: (self, args, ctx) => {
        return ctx.openVault.apps.getAppsForUser(self.localID);
      }
    },
    defaultEthAddress: {
      type: _graphql.GraphQLString,
      resolve: (self, args, ctx) => {
        return ctx.queries.getUserDefaultEthAccount(self.localID);
      }
    },
    wallets: {
      type: new _graphql.GraphQLNonNull(ethWallets),
      resolve: (self, args, ctx) => {
        return ctx.queries.getUserEthWallets(self.localID);
      }
    },
    profile: {
      type: new _graphql.GraphQLNonNull(namedProfile)
    },
    pubKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    privateProfile: {
      type: _graphql.GraphQLBoolean
    }
  })
});
exports.ownUserIdentity = ownUserIdentity;
const peerAppIdentity = new _graphql.GraphQLObjectType({
  name: 'PeerAppIdentity',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    pubKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  })
});
exports.peerAppIdentity = peerAppIdentity;
const peerDeveloperIdentity = new _graphql.GraphQLObjectType({
  name: 'PeerDeveloperIdentity',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    pubKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    }
  })
});
exports.peerDeveloperIdentity = peerDeveloperIdentity;
const peerUserIdentity = new _graphql.GraphQLObjectType({
  name: 'PeerUserIdentity',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    pubKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    profile: {
      type: new _graphql.GraphQLNonNull(genericProfile)
    }
  })
});
exports.peerUserIdentity = peerUserIdentity;
const peer = new _graphql.GraphQLObjectType({
  name: 'Peer',
  fields: () => ({
    publicKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    publicFeed: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    profile: {
      type: new _graphql.GraphQLNonNull(genericProfile)
    }
  })
});
exports.peer = peer;
const connectionState = new _graphql.GraphQLEnumType({
  name: 'ConnectionState',
  values: {
    CONNECTED: {
      value: 'connected'
    },
    SENDING: {
      value: 'sending'
    },
    SENT: {
      value: 'sent'
    }
  }
});
exports.connectionState = connectionState;
const contact = new _graphql.GraphQLObjectType({
  name: 'Contact',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    peerID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    publicFeed: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    pubKey: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    profile: {
      type: new _graphql.GraphQLNonNull(genericProfile),
      resolve: (self, args, ctx) => {
        return ctx.openVault.identities.getContactProfile(self.localID);
      }
    },
    connectionState: {
      type: new _graphql.GraphQLNonNull(connectionState)
    }
  })
});
exports.contact = contact;
const contacts = new _graphql.GraphQLObjectType({
  name: 'Contacts',
  fields: () => ({
    userContacts: {
      type: new _graphql.GraphQLList(contact),
      args: {
        userID: {
          type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
        }
      },
      resolve: (self, args, ctx) => {
        return ctx.queries.getUserContacts(args.userID);
      }
    }
  })
});
exports.contacts = contacts;
const apps = new _graphql.GraphQLObjectType({
  name: 'Apps',
  fields: () => ({
    installed: {
      type: new _graphql.GraphQLList(app),
      resolve: (self, args, ctx) => {
        return Object.values(ctx.openVault.apps.apps);
      }
    },
    own: {
      type: new _graphql.GraphQLList(ownApp),
      resolve: (self, args, ctx) => {
        return Object.values(ctx.openVault.apps.ownApps);
      }
    }
  })
});
exports.apps = apps;
const identities = new _graphql.GraphQLObjectType({
  name: 'Identities',
  fields: () => ({
    ownUsers: {
      type: new _graphql.GraphQLList(ownUserIdentity),
      resolve: (self, args, ctx) => {
        return Object.values(ctx.openVault.identities.ownUsers);
      }
    },
    ownDevelopers: {
      type: new _graphql.GraphQLList(ownDeveloperIdentity),
      resolve: (self, args, ctx) => {
        return Object.values(ctx.openVault.identities.ownDevelopers);
      }
    }
  })
});
exports.identities = identities;
const peers = new _graphql.GraphQLObjectType({
  name: 'Peers',
  fields: () => ({
    peerLookupByFeed: {
      type: peer,
      args: {
        feedHash: {
          type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
        }
      },
      resolve: async (self, args, ctx) => {
        try {
          const peerPublicRes = await ctx.io.bzz.download(args.feedHash);
          const data = await peerPublicRes.json();
          return {
            profile: data.profile,
            publicFeed: args.feedHash,
            publicKey: data.publicKey
          };
        } catch (err) {
          ctx.log(err);
          return null;
        }
      }
    }
  })
});
exports.peers = peers;
const walletBalances = new _graphql.GraphQLObjectType({
  name: 'WalletBalances',
  fields: () => ({
    eth: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: async (self, args, ctx) => {
        try {
          return await ctx.io.eth.getETHBalance(self);
        } catch (err) {
          ctx.log(err);
          return 0;
        }
      }
    },
    mft: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: async (self, args, ctx) => {
        try {
          return await ctx.io.eth.getMFTBalance(self);
        } catch (err) {
          ctx.log(err);
          return 0;
        }
      }
    }
  })
});
exports.walletBalances = walletBalances;
const walletAccount = new _graphql.GraphQLObjectType({
  name: 'WalletAccount',
  fields: () => ({
    address: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: self => self
    },
    balances: {
      type: new _graphql.GraphQLNonNull(walletBalances),
      resolve: self => self
    }
  })
});
exports.walletAccount = walletAccount;
const ethLedgerWallet = new _graphql.GraphQLObjectType({
  name: 'EthLedgerWallet',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    name: {
      type: _graphql.GraphQLString
    },
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    accounts: {
      type: new _graphql.GraphQLList(walletAccount)
    }
  })
});
exports.ethLedgerWallet = ethLedgerWallet;
const ethHdWallet = new _graphql.GraphQLObjectType({
  name: 'EthHDWallet',
  interfaces: () => [nodeInterface],
  fields: () => ({
    id: idResolver,
    name: {
      type: _graphql.GraphQLString
    },
    localID: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
    },
    mnemonic: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
    },
    accounts: {
      type: new _graphql.GraphQLList(walletAccount)
    }
  })
});
exports.ethHdWallet = ethHdWallet;
const ethWallets = new _graphql.GraphQLObjectType({
  name: 'EthWallets',
  fields: () => ({
    hd: {
      type: new _graphql.GraphQLList(ethHdWallet),
      resolve: self => {
        return self.filter(w => w.type === 'hd');
      }
    },
    ledger: {
      type: new _graphql.GraphQLList(ethLedgerWallet),
      resolve: self => {
        return self.filter(w => w.type === 'ledger');
      }
    }
  })
});
exports.ethWallets = ethWallets;
const wallets = new _graphql.GraphQLObjectType({
  name: 'Wallets',
  fields: () => ({
    ethWallets: {
      type: new _graphql.GraphQLNonNull(ethWallets),
      args: {
        userID: {
          type: new _graphql.GraphQLNonNull(_graphql.GraphQLString)
        }
      },
      resolve: (self, args, ctx) => {
        return ctx.queries.getUserEthWallets(args.userID);
      }
    }
  })
});
exports.wallets = wallets;
const settings = new _graphql.GraphQLObjectType({
  name: 'Settings',
  fields: () => ({
    ethereumUrl: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
      resolve: (self, args, ctx) => {
        return ctx.openVault.settings.ethURL;
      }
    }
  })
});
exports.settings = settings;
const viewer = new _graphql.GraphQLObjectType({
  name: 'Viewer',
  fields: () => ({
    id: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      resolve: () => 'viewer'
    },
    apps: {
      type: new _graphql.GraphQLNonNull(apps),
      resolve: () => ({})
    },
    contacts: {
      type: new _graphql.GraphQLNonNull(contacts),
      resolve: () => ({})
    },
    identities: {
      type: new _graphql.GraphQLNonNull(identities),
      resolve: () => ({})
    },
    wallets: {
      type: new _graphql.GraphQLNonNull(wallets),
      resolve: () => ({})
    },
    settings: {
      type: new _graphql.GraphQLNonNull(settings),
      resolve: () => ({})
    }
  })
});
exports.viewer = viewer;