"use strict";

exports.__esModule = true;
exports.default = void 0;

// Adapted from https://github.com/graphql/graphql-js/blob/master/src/subscription/__tests__/eventEmitterAsyncIterator.js
const observableToAsyncIterator = (observable, dispose) => {
  const pullQueue = [];
  const pushQueue = [];
  let listening = true;
  const subscription = observable.subscribe(value => {
    if (pullQueue.length !== 0) {
      pullQueue.shift()({
        value,
        done: false
      });
    } else {
      pushQueue.push(value);
    }
  });

  const pullValue = () => {
    return new Promise(resolve => {
      if (pushQueue.length !== 0) {
        const value = pushQueue.shift();
        resolve({
          value,
          done: false
        });
      } else {
        pullQueue.push(resolve);
      }
    });
  };

  const emptyQueue = () => {
    if (listening) {
      listening = false;
      subscription.unsubscribe();

      for (const resolve of pullQueue) {
        resolve({
          value: undefined,
          done: true
        });
      }

      pullQueue.length = 0;
      pushQueue.length = 0;

      if (dispose != null) {
        dispose();
      }
    }
  }; // $FlowFixMe: AsyncIterator support


  return {
    // $FlowFixMe: AsyncIterator support
    [Symbol.asyncIterator]() {
      return this;
    },

    next() {
      return listening ? pullValue() : this.return();
    },

    return() {
      emptyQueue();
      return Promise.resolve({
        value: undefined,
        done: true
      });
    },

    throw(error) {
      emptyQueue();
      return Promise.reject(error);
    }

  };
};

var _default = observableToAsyncIterator;
exports.default = _default;