"use strict";

exports.__esModule = true;
exports.default = void 0;

var _utilsId = require("@mainframe/utils-id");

var _feed = require("../swarm/feed");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Contact {
  static _generateContactPayload() {
    return {
      version: '1.0.0',
      apps: {}
    };
  }

  constructor(localID, peerID, profile, aliasName, sharedFeed, requestSent) {
    _defineProperty(this, "_localID", void 0);

    _defineProperty(this, "_peerID", void 0);

    _defineProperty(this, "_sharedFeed", void 0);

    _defineProperty(this, "_requestSent", void 0);

    _defineProperty(this, "_aliasName", void 0);

    _defineProperty(this, "_profile", void 0);

    this._localID = localID;
    this._peerID = peerID;
    this._profile = profile;
    this._aliasName = aliasName;
    this._sharedFeed = sharedFeed;
    this._requestSent = !!requestSent;
  }

  get localID() {
    return this._localID;
  }

  get peerID() {
    return this._peerID;
  }

  get sharedFeed() {
    return this._sharedFeed;
  }

  get profile() {
    return this._profile;
  }

  set profile(profile) {
    this._profile = profile;
  }

  get name() {
    return this._aliasName || this._profile.name;
  }

  set aliasName(aliasName) {
    this._aliasName = aliasName;
  }

  set requestSent(requestSent) {
    this._requestSent = requestSent;
  }

  get connectionState() {
    if (!this._requestSent) return 'sending';
    return this.sharedFeed.remoteFeed ? 'connected' : 'sent';
  }

  generatefirstContactPayload() {
    if (!this.sharedFeed.localFeed.feedHash) {
      throw new Error('Contact feed manifest has not been synced');
    }

    return {
      version: '1.0.0',
      privateFeed: this.sharedFeed.localFeed.feedHash
    };
  }

}

exports.default = Contact;

_defineProperty(Contact, "create", (peerID, optional) => {
  const sharedFeed = _feed.BidirectionalFeed.create({
    remoteFeed: optional && optional.contactFeed
  });

  sharedFeed.localFeedData = Contact._generateContactPayload();
  return new Contact(optional && optional.localID || (0, _utilsId.uniqueID)(), peerID, optional && optional.profile || {}, optional && optional.aliasName, sharedFeed, false);
});

_defineProperty(Contact, "fromJSON", contactSerialized => new Contact(contactSerialized.localID, contactSerialized.peerID, contactSerialized.profile, contactSerialized.aliasName, _feed.BidirectionalFeed.fromJSON(contactSerialized.sharedFeed), contactSerialized.requestSent));

_defineProperty(Contact, "toJSON", contact => ({
  localID: contact.localID,
  peerID: contact.peerID,
  profile: contact.profile,
  aliasName: contact._aliasName,
  sharedFeed: _feed.BidirectionalFeed.toJSON(contact.sharedFeed),
  requestSent: contact._requestSent
}));