"use strict";

exports.__esModule = true;
exports.default = void 0;

var _dataTypes = require("@mainframe/data-types");

var _utilsId = require("@mainframe/utils-id");

var _multibase = _interopRequireDefault(require("multibase"));

var _utils = require("../utils");

var _AppIdentity = _interopRequireDefault(require("./AppIdentity"));

var _DeveloperIdentity = _interopRequireDefault(require("./DeveloperIdentity"));

var _OwnAppIdentity = _interopRequireDefault(require("./OwnAppIdentity"));

var _OwnDeveloperIdentity = _interopRequireDefault(require("./OwnDeveloperIdentity"));

var _OwnUserIdentity = _interopRequireDefault(require("./OwnUserIdentity"));

var _PeerUserIdentity = _interopRequireDefault(require("./PeerUserIdentity"));

var _Contact = _interopRequireDefault(require("./Contact"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const getReference = identity => {
  if (identity instanceof _AppIdentity.default) {
    return {
      domain: 'apps',
      ownership: 'peers'
    };
  }

  if (identity instanceof _DeveloperIdentity.default) {
    return {
      domain: 'developers',
      ownership: 'peers'
    };
  }

  if (identity instanceof _PeerUserIdentity.default) {
    return {
      domain: 'users',
      ownership: 'peers'
    };
  }

  if (identity instanceof _OwnAppIdentity.default) {
    return {
      domain: 'apps',
      ownership: 'own'
    };
  }

  if (identity instanceof _OwnDeveloperIdentity.default) {
    return {
      domain: 'developers',
      ownership: 'own'
    };
  }

  if (identity instanceof _OwnUserIdentity.default) {
    return {
      domain: 'users',
      ownership: 'own'
    };
  }
};

const fromAppIdentity = (0, _utils.mapObject)(_AppIdentity.default.toJSON);
const fromDeveloperIdentity = (0, _utils.mapObject)(_DeveloperIdentity.default.toJSON);
const fromPeerUserIdentity = (0, _utils.mapObject)(_PeerUserIdentity.default.toJSON);

const fromPeersGroup = group => ({
  // $FlowFixMe: mapping type
  apps: fromAppIdentity(group.apps),
  // $FlowFixMe: mapping type
  developers: fromDeveloperIdentity(group.developers),
  // $FlowFixMe: mapping type
  users: fromPeerUserIdentity(group.users)
});

const fromOwnAppIdentity = (0, _utils.mapObject)(_OwnAppIdentity.default.toJSON);
const fromOwnDeveloperIdentity = (0, _utils.mapObject)(_OwnDeveloperIdentity.default.toJSON);
const fromOwnUserIdentity = (0, _utils.mapObject)(_OwnUserIdentity.default.toJSON);

const fromOwnGroup = group => ({
  // $FlowFixMe: mapping type
  apps: fromOwnAppIdentity(group.apps),
  // $FlowFixMe: mapping type
  developers: fromOwnDeveloperIdentity(group.developers),
  // $FlowFixMe: mapping type
  users: fromOwnUserIdentity(group.users)
});

const toAppIdentity = (0, _utils.mapObject)(_AppIdentity.default.fromJSON);
const toDeveloperIdentity = (0, _utils.mapObject)(_DeveloperIdentity.default.fromJSON);
const toPeerUserIdentity = (0, _utils.mapObject)(_PeerUserIdentity.default.fromJSON);

const toPeersGroup = group => ({
  // $FlowFixMe: mapping type
  apps: toAppIdentity(group.apps),
  // $FlowFixMe: mapping type
  developers: toDeveloperIdentity(group.developers),
  // $FlowFixMe: mapping type
  users: toPeerUserIdentity(group.users)
});

const toOwnAppIdentity = (0, _utils.mapObject)(_OwnAppIdentity.default.fromJSON);
const toOwnDeveloperIdentity = (0, _utils.mapObject)(_OwnDeveloperIdentity.default.fromJSON);
const toOwnUserIdentity = (0, _utils.mapObject)(_OwnUserIdentity.default.fromJSON);

const toOwnGroup = group => ({
  // $FlowFixMe: mapping type
  apps: toOwnAppIdentity(group.apps),
  // $FlowFixMe: mapping type
  developers: toOwnDeveloperIdentity(group.developers),
  // $FlowFixMe: mapping type
  users: toOwnUserIdentity(group.users)
});

const createPeersGroup = (params = {}) => ({
  apps: params.apps == null ? {} : params.apps,
  developers: params.developers == null ? {} : params.developers,
  users: params.users == null ? {} : params.users
});

const createOwnGroup = (params = {}) => ({
  apps: params.apps == null ? {} : params.apps,
  developers: params.developers == null ? {} : params.developers,
  users: params.users == null ? {} : params.users
});

const contactsFromJSON = (0, _utils.mapObject)(_Contact.default.fromJSON);
const contactsToJSON = (0, _utils.mapObject)(_Contact.default.toJSON);

const toContacts = (params = {}) => {
  return Object.keys(params).reduce((acc, ownID) => {
    // $FlowFixMe: mapper type
    acc[ownID] = contactsFromJSON(params[ownID]);
    return acc;
  }, {});
};

const fromContacts = contacts => {
  return Object.keys(contacts).reduce((acc, ownID) => {
    // $FlowFixMe: mapper type
    acc[ownID] = contactsToJSON(contacts[ownID]);
    return acc;
  }, {});
};

const truncateString = string => {
  if (string.length < 20) {
    return string;
  }

  const start = string.substr(0, 7);
  const end = string.substr(string.length - 7, string.length);
  return `${start}...${end}`;
};

class IdentitiesRepository {
  constructor(identities = {}) {
    _defineProperty(this, "_byMFID", {});

    _defineProperty(this, "_identities", void 0);

    _defineProperty(this, "_refs", {});

    _defineProperty(this, "_mfidByFeed", {});

    _defineProperty(this, "_userByContact", {});

    this._identities = {
      own: createOwnGroup(identities.own),
      peers: createPeersGroup(identities.peers),
      contacts: identities.contacts || {}
    };
    Object.keys(this._identities).forEach(ownership => {
      if (ownership === 'contacts') {
        // Add references so own users can be looked up by contact
        Object.keys(this._identities.contacts).forEach(ownId => {
          Object.keys(this._identities.contacts[ownId]).forEach(cid => {
            this._userByContact[(0, _utilsId.idType)(cid)] = (0, _utilsId.idType)(ownId);
          });
        });
      } else {
        // Add references so an identity can be retrieved by local ID or Mainframe ID lookup
        Object.keys(this._identities[ownership]).forEach(domain => {
          Object.keys(this._identities[ownership][domain]).forEach(id => {
            const identity = this._identities[ownership][domain][id];

            const mfid = _dataTypes.MFID.canonical(identity.id);

            this._byMFID[mfid] = (0, _utilsId.idType)(id);
            this._refs[id] = {
              ownership,
              domain
            };

            if (identity instanceof _PeerUserIdentity.default) {
              this._mfidByFeed[identity.publicFeed] = mfid;
            }
          });
        });
      }
    });
  } // Getters


  get ownIdentities() {
    return this._identities.own;
  }

  get peerIdentities() {
    return this._identities.peers;
  }

  get ownApps() {
    return this._identities.own.apps;
  }

  get ownDevelopers() {
    return this._identities.own.developers;
  }

  get ownUsers() {
    return this._identities.own.users;
  }

  get peerApps() {
    return this._identities.peers.apps;
  }

  get peerDevelopers() {
    return this._identities.peers.developers;
  }

  get peerUsers() {
    return this._identities.peers.users;
  }

  get contacts() {
    return this._identities.contacts;
  }

  getContactsForUser(userID) {
    return this._identities.contacts[userID];
  }

  getContact(userID, contactID) {
    return this._identities.contacts[userID][contactID];
  }

  getOwnApp(id) {
    return this._identities.own.apps[id];
  }

  getOwnDeveloper(id) {
    return this._identities.own.developers[id];
  }

  getOwnUser(id) {
    return this._identities.own.users[id];
  }

  getPeerApp(id) {
    return this._identities.peers.apps[id];
  }

  getPeerDeveloper(id) {
    return this._identities.peers.developers[id];
  }

  getPeerUser(id) {
    return this._identities.peers.users[id];
  }

  getApp(id) {
    return this.getOwnApp(id) || this.getPeerApp(id);
  }

  getDeveloper(id) {
    return this.getOwnDeveloper(id) || this.getPeerDeveloper(id);
  }

  getUser(id) {
    return this.getOwnUser(id) || this.getPeerUser(id);
  }

  getOwn(id) {
    return this.getOwnApp(id) || this.getOwnDeveloper(id) || this.getOwnUser(id);
  }

  getPeer(id) {
    return this.getPeerApp(id) || this.getPeerDeveloper(id) || this.getPeerUser(id);
  }

  getPeerByFeed(publicFeed) {
    if (this._mfidByFeed[publicFeed]) {
      return this.getPeerUser(this._byMFID[this._mfidByFeed[publicFeed]]);
    }
  }

  getIdentity(id) {
    const ref = this._refs[id];

    if (ref != null && ref.ownership !== 'contacts') {
      return this._identities[ref.ownership][ref.domain][id];
    }
  }

  getID(mfid) {
    return this._byMFID[_dataTypes.MFID.canonical(mfid)];
  }

  getByMFID(mfid) {
    const id = this.getID(mfid);

    if (id != null) {
      return this.getIdentity(id);
    }
  } // Setters


  addIdentity(identity) {
    const foundID = this._byMFID[identity.id];

    if (foundID != null) {
      return foundID;
    }

    const ref = getReference(identity);

    if (ref == null) {
      throw new Error('Unsupported identity');
    }

    this._byMFID[identity.id] = (0, _utilsId.idType)(identity.localID);
    this._refs[identity.localID] = ref; // $FlowFixMe: polymorphic type

    this._identities[ref.ownership][ref.domain][identity.localID] = identity;
    return (0, _utilsId.idType)(identity.localID);
  }

  createOwnApp(keyPair) {
    return this.addIdentity(_OwnAppIdentity.default.create(keyPair));
  }

  createOwnDeveloper(profile, keyPair) {
    const id = this.addIdentity(_OwnDeveloperIdentity.default.create(profile, keyPair));
    const dev = this.getOwnDeveloper(id);
    if (dev) return dev;else throw new Error('Error creating developer');
  }

  createOwnUser(profile, privateProfile, keyPair) {
    const id = this.addIdentity(_OwnUserIdentity.default.create(profile, privateProfile, keyPair));
    const user = this.getOwnUser(id);
    if (user) return user;else throw new Error('Error creating user');
  }

  createPeerApp(key) {
    return this.addIdentity(new _AppIdentity.default((0, _utilsId.uniqueID)(), key));
  }

  createPeerDeveloper(key, profile) {
    return this.addIdentity(new _DeveloperIdentity.default((0, _utilsId.uniqueID)(), key, profile));
  }

  createPeerUser(publicKey, profile, publicFeed, firstContactAddress, feeds) {
    if (this._mfidByFeed[publicFeed]) {
      const id = this._byMFID[this._mfidByFeed[publicFeed]];
      const identity = this.getIdentity(id);

      if (!(identity instanceof _PeerUserIdentity.default)) {
        throw new Error('Error adding peer - identity already exists with publicFeed');
      }

      return identity;
    }

    const keyBuffer = _multibase.default.decode(publicKey);

    const id = this.addIdentity(new _PeerUserIdentity.default((0, _utilsId.uniqueID)(), keyBuffer, profile, publicFeed, firstContactAddress, feeds));
    const peer = this.getPeerUser(id);
    if (!peer) throw new Error('Error adding peer');
    this._mfidByFeed[String(publicFeed)] = peer.id;
    return peer;
  }

  createContactFromPeer(ownUserId, peerID, aliasName) {
    const peer = this.getPeerUser((0, _utilsId.idType)(peerID));
    if (!peer) throw new Error('Peer not found');

    if (this._identities.contacts[ownUserId]) {
      const keys = Object.keys(this._identities.contacts[ownUserId]);
      const contacts = keys.map(id => this._identities.contacts[ownUserId][id]);
      const existing = contacts.find(c => c.peerID === peerID);

      if (existing) {
        return existing;
      }
    }

    const contact = _Contact.default.create(peerID, {
      aliasName
    });

    if (this._identities.contacts[ownUserId]) {
      this._identities.contacts[ownUserId][contact.localID] = contact;
    } else {
      this._identities.contacts[ownUserId] = {
        [contact.localID]: contact
      };
    }

    this._userByContact[contact.localID] = ownUserId;
    return contact;
  }

  deleteContact(userID, contactID) {
    if (this._identities.contacts[userID]) {
      delete this._identities.contacts[userID][contactID];
    }

    if (this._userByContact[contactID]) {
      delete this._userByContact[contactID];
    }
  }

  getContactProfile(contactID) {
    const userID = this._userByContact[contactID];

    if (userID == null) {
      return {};
    }

    const userContacts = this._identities.contacts[userID];

    if (userContacts == null) {
      return {};
    }

    const contact = userContacts[contactID];

    if (contact == null) {
      return {};
    }

    const peer = this._identities.peers.users[contact.peerID];
    const peerProfile = peer ? peer.profile : {};
    return {
      name: contact.name || peerProfile.name || truncateString(peer.publicFeed),
      avatar: contact.profile.avatar || peerProfile.avatar,
      ethAddress: contact.profile.ethAddress || peerProfile.ethAddress
    };
  } // TODO: Refactor contacts storage to lookup by ID, store userID in contact


  getContactByID(contactID) {
    const userID = this._userByContact[contactID];
    if (userID == null) return;
    const contact = this._identities.contacts[userID][contactID];
    if (contact == null) return;
    return {
      userID,
      contact
    };
  }

}

exports.default = IdentitiesRepository;

_defineProperty(IdentitiesRepository, "fromJSON", (serialized = {}) => {
  return new IdentitiesRepository({
    // $FlowFixMe: mapper type
    own: serialized.own ? toOwnGroup(serialized.own) : {},
    // $FlowFixMe: mapper type
    peers: serialized.peers ? toPeersGroup(serialized.peers) : {},
    contacts: serialized.contacts ? toContacts(serialized.contacts) : {}
  });
});

_defineProperty(IdentitiesRepository, "toJSON", repository => ({
  // $FlowFixMe: mapper type
  own: fromOwnGroup(repository.ownIdentities),
  // $FlowFixMe: mapper type
  peers: fromPeersGroup(repository.peerIdentities),
  contacts: fromContacts(repository.contacts)
}));