"use strict";

exports.__esModule = true;
exports.default = void 0;

var _dataTypes = require("@mainframe/data-types");

var _utilsCrypto = require("@mainframe/utils-crypto");

var _multibase = _interopRequireDefault(require("multibase"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const KEY_TYPE = 'pub-key:ed25519';

class Identity {
  constructor(localID, idType, keyOrID) {
    _defineProperty(this, "_id", void 0);

    _defineProperty(this, "_localID", void 0);

    _defineProperty(this, "_key", void 0);

    const type = `${idType}:${KEY_TYPE}`;
    this._localID = localID;

    if (typeof keyOrID === 'string') {
      const id = new _dataTypes.MFID(keyOrID);

      if (id.type.value !== type) {
        throw new Error(`Invalid Mainframe ID type: expected ${type}, got ${id.type.value}`);
      }

      this._id = id.toString();
      this._key = id.data.toBuffer();
    } else {
      this._key = keyOrID;
      this._id = _dataTypes.MFID.create(type, this._key).toString();
    }
  }

  get id() {
    return this._id;
  }

  get localID() {
    return this._localID;
  }

  get publicKey() {
    return this._key;
  }

  base64PublicKey() {
    return _multibase.default.encode('base64', this.publicKey).toString();
  }

  verifySignature(message, signature) {
    return (0, _utilsCrypto.verifySignature)(message, signature, this._key);
  }

  openSigned(signed) {
    return (0, _utilsCrypto.openSigned)(signed, this._key);
  }

}

exports.default = Identity;