"use strict";

exports.__esModule = true;
exports.default = void 0;

var _utilsBase = require("@mainframe/utils-base64");

var _utilsCrypto = require("@mainframe/utils-crypto");

var _utilsId = require("@mainframe/utils-id");

var _utils = require("../utils");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const parseKeyPair = serialized => ({
  publicKey: (0, _utilsBase.decodeBase64)(serialized.publicKey),
  secretKey: (0, _utilsBase.decodeBase64)(serialized.secretKey)
});

const serializeKeyPair = keyPair => ({
  publicKey: (0, _utilsBase.encodeBase64)(keyPair.publicKey),
  secretKey: (0, _utilsBase.encodeBase64)(keyPair.secretKey)
});

const fromKeysPairs = (0, _utils.mapObject)(serializeKeyPair);
const fromKeysBuffers = (0, _utils.mapObject)(_utilsBase.encodeBase64);
const toKeysPairs = (0, _utils.mapObject)(parseKeyPair);
const toKeysBuffers = (0, _utils.mapObject)(_utilsBase.decodeBase64);

class Keychain {
  constructor(keys = {}) {
    _defineProperty(this, "_keys", void 0);

    _defineProperty(this, "_refs", void 0);

    this._keys = {
      pairBox: keys.pairBox == null ? {} : keys.pairBox,
      pairSign: keys.pairSign == null ? {} : keys.pairSign,
      publicBox: keys.publicBox == null ? {} : keys.publicBox,
      publicSign: keys.publicSign == null ? {} : keys.publicSign,
      secretBox: keys.secretBox == null ? {} : keys.secretBox
    };
    this._refs = Object.keys(this._keys).reduce((acc, type) => {
      Object.keys(this._keys[type]).forEach(key => acc[key] = type);
      return acc;
    }, {});
  }

  get pairBox() {
    return this._keys.pairBox;
  }

  get pairSign() {
    return this._keys.pairSign;
  }

  get publicBox() {
    return this._keys.publicBox;
  }

  get publicSign() {
    return this._keys.publicSign;
  }

  get secretBox() {
    return this._keys.secretBox;
  }

  addKey(type, key) {
    const id = (0, _utilsId.uniqueID)();
    this._refs[id] = type;
    this._keys[type][id] = key;
    return id;
  }

  addKeyPair(type, keyPair) {
    const id = (0, _utilsId.uniqueID)();
    this._refs[id] = type;
    this._keys[type][id] = keyPair;
    return id;
  }

  addPairBox(keyPair) {
    return this.addKeyPair('pairBox', keyPair);
  }

  addPairSign(keyPair) {
    return this.addKeyPair('pairSign', keyPair);
  }

  addPublicBox(key) {
    return this.addKey('publicBox', key);
  }

  addPublicSign(key) {
    return this.addKey('publicSign', key);
  }

  addSecretBox(key) {
    return this.addKey('secretBox', key);
  }

  createPairBox(seed) {
    return this.addPairBox((0, _utilsCrypto.createBoxKeyPair)(seed));
  }

  createPairSign(seed) {
    return this.addPairSign((0, _utilsCrypto.createSignKeyPair)(seed));
  }

  createSecretBox() {
    return this.addSecretBox((0, _utilsCrypto.createSecretBoxKey)());
  }

  getPairBox(id) {
    return id == null ? // $FlowFixMe: return type
    Object.values(this._keys.pairBox)[0] : this._keys.pairBox[id];
  }

  getPairSign(id) {
    return id == null ? // $FlowFixMe: return type
    Object.values(this._keys.pairSign)[0] : this._keys.pairSign[id];
  }

  getPublicBox(id) {
    return id == null ? // $FlowFixMe: return type
    Object.values(this._keys.publicBox)[0] : this._keys.publicBox[id];
  }

  getPublicSign(id) {
    return id == null ? // $FlowFixMe: return type
    Object.values(this._keys.publicSign)[0] : this._keys.publicSign[id];
  }

  getSecretBox(id) {
    return id == null ? // $FlowFixMe: return type
    Object.values(this._keys.secretBox)[0] : this._keys.secretBox[id];
  }

  removeKey(id) {
    const type = this._refs[id];

    if (type == null) {
      return false;
    }

    delete this._keys[type][id];
    delete this._refs[id];
    return true;
  }

}

exports.default = Keychain;

_defineProperty(Keychain, "fromJSON", serialized => {
  return new Keychain({
    // $FlowFixMe: mapping types
    pairBox: toKeysPairs(serialized.pairBox),
    // $FlowFixMe: mapping types
    pairSign: toKeysPairs(serialized.pairSign),
    // $FlowFixMe: mapping types
    publicBox: toKeysBuffers(serialized.publicBox),
    // $FlowFixMe: mapping types
    publicSign: toKeysBuffers(serialized.publicSign),
    // $FlowFixMe: mapping types
    secretBox: toKeysBuffers(serialized.secretBox)
  });
});

_defineProperty(Keychain, "toJSON", keychain => ({
  // $FlowFixMe: mapping types
  pairBox: fromKeysPairs(keychain.pairBox),
  // $FlowFixMe: mapping types
  pairSign: fromKeysPairs(keychain.pairSign),
  // $FlowFixMe: mapping types
  publicBox: fromKeysBuffers(keychain.publicBox),
  // $FlowFixMe: mapping types
  publicSign: fromKeysBuffers(keychain.publicSign),
  // $FlowFixMe: mapping types
  secretBox: fromKeysBuffers(keychain.secretBox)
}));