"use strict";

exports.__esModule = true;
exports.default = exports.serializeKeyPair = exports.parseKeyPair = void 0;

var _utilsCrypto = require("@mainframe/utils-crypto");

var _multibase = _interopRequireDefault(require("multibase"));

var _Identity = _interopRequireDefault(require("./Identity"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const parseKeyPair = serialized => ({
  publicKey: _multibase.default.decode(serialized.publicKey),
  secretKey: _multibase.default.decode(serialized.secretKey)
});

exports.parseKeyPair = parseKeyPair;

const serializeKeyPair = keyPair => ({
  publicKey: _multibase.default.encode('base64', keyPair.publicKey).toString(),
  secretKey: _multibase.default.encode('base64', keyPair.secretKey).toString()
});

exports.serializeKeyPair = serializeKeyPair;

class OwnIdentity extends _Identity.default {
  constructor(localID, type, keyPair) {
    super(localID, type, keyPair.publicKey);

    _defineProperty(this, "_keyPair", void 0);

    this._keyPair = keyPair;
  }

  get keyPair() {
    return this._keyPair;
  }

  getSignature(message) {
    return (0, _utilsCrypto.getSignature)(message, this._keyPair.secretKey);
  }

  sign(message) {
    return (0, _utilsCrypto.sign)(message, this._keyPair.secretKey);
  }

}

exports.default = OwnIdentity;