"use strict";

exports.__esModule = true;
exports.default = void 0;

var _net = require("net");

var _config = require("@mainframe/config");

var _fsExtra = require("fs-extra");

var _vault = require("../vault");

var _handleClient = _interopRequireDefault(require("./handleClient"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class ServerHandler {
  constructor(env) {
    _defineProperty(this, "_clients", new Set());

    _defineProperty(this, "_path", void 0);

    _defineProperty(this, "_server", void 0);

    _defineProperty(this, "_vaults", new _vault.VaultRegistry());

    this._path = (0, _config.getDaemonSocketPath)(env);
    this._server = (0, _net.createServer)(socket => {
      this._clients.add(socket);

      socket.on('close', async () => {
        await this._vaults.close(socket);

        this._clients.delete(socket);
      });
      (0, _handleClient.default)(socket, env, this._vaults);
    });

    this._server.on('close', () => {
      this._clients.clear();
    });
  }

  get listening() {
    // $FlowFixMe: missing property in type definition?
    return this._server.listening;
  }

  async start() {
    if (this.listening) {
      return Promise.resolve();
    }

    await (0, _fsExtra.remove)(this._path);
    return new Promise((resolve, reject) => {
      this._server.listen(this._path, err => {
        if (err) reject(err);else resolve();
      });
    });
  }

  async stop() {
    if (!this.listening) {
      return Promise.resolve();
    }

    return new Promise(resolve => {
      this._server.close(resolve);

      this._clients.forEach(socket => {
        // $FlowFixMe: missing type def for destroyed?
        if (!socket.destroyed) {
          socket.destroy();
        }
      });
    });
  }

}

exports.default = ServerHandler;