"use strict";

exports.__esModule = true;
exports.permissionErrorFromResult = exports.permissionDeniedError = exports.permissionInputError = exports.permissionNotSetError = exports.permissionUnknowError = exports.permissionError = exports.sessionError = exports.clientError = exports.vaultError = exports.daemonError = void 0;

var _rpcError = _interopRequireDefault(require("@mainframe/rpc-error"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Mainframe protocol
const daemonError = message => new _rpcError.default(1000, message);

exports.daemonError = daemonError;

const vaultError = message => new _rpcError.default(2000, message);

exports.vaultError = vaultError;

const clientError = message => new _rpcError.default(3000, message);

exports.clientError = clientError;

const sessionError = message => new _rpcError.default(4000, message); // Permission errors (session-level)


exports.sessionError = sessionError;

const permissionError = (message = 'Permission error') => {
  return new _rpcError.default(4100, message);
};

exports.permissionError = permissionError;

const permissionUnknowError = (message = 'Unknow permission') => {
  return new _rpcError.default(4101, message);
};

exports.permissionUnknowError = permissionUnknowError;

const permissionNotSetError = (message = 'Permission not set') => {
  return new _rpcError.default(4102, message);
};

exports.permissionNotSetError = permissionNotSetError;

const permissionInputError = (message = 'Invalid input') => {
  return new _rpcError.default(4103, message);
};

exports.permissionInputError = permissionInputError;

const permissionDeniedError = (message = 'Permission denied') => {
  return new _rpcError.default(4104, message);
};

exports.permissionDeniedError = permissionDeniedError;

const permissionErrorFromResult = result => {
  switch (result) {
    case 'unknown_key':
      return permissionUnknowError();

    case 'not_set':
      return permissionNotSetError();

    case 'invalid_input':
      return permissionInputError();

    case 'denied':
      return permissionDeniedError();

    default:
      return permissionError();
  }
};

exports.permissionErrorFromResult = permissionErrorFromResult;