"use strict";

exports.__esModule = true;
exports.default = void 0;

var _util = require("util");

var _appManifest = require("@mainframe/app-manifest");

var _rpcError = require("@mainframe/rpc-error");

var _rpcHandler = _interopRequireDefault(require("@mainframe/rpc-handler"));

var _utilsId = require("@mainframe/utils-id");

var _debug = _interopRequireDefault(require("debug"));

var _oboe = _interopRequireDefault(require("oboe"));

var _ClientContext = _interopRequireDefault(require("../context/ClientContext"));

var _methods = _interopRequireDefault(require("./methods"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const handleMessage = (0, _rpcHandler.default)({
  methods: _methods.default,
  onHandlerError: (ctx, msg, err) => {
    ctx.log('handler error', msg, err.stack);
  },
  onInvalidMessage: (ctx, msg) => {
    ctx.log('invalid message', msg);
  },
  onNotification: (ctx, msg) => {
    ctx.log('notification received', msg);
  },
  validatorOptions: {
    messages: _appManifest.MANIFEST_SCHEMA_MESSAGES
  }
});

var _default = (socket, env, vaults) => {
  const ns = `mainframe:daemon:rpc:client:${(0, _utilsId.uniqueID)()}`;
  const log = (0, _debug.default)(ns);
  const logIO = (0, _debug.default)(`${ns}:io`);

  const logJSON = (msg, data) => {
    logIO(msg, (0, _util.inspect)(data, {
      colors: true,
      depth: 5
    }));
  };

  const sendJSON = payload => {
    logJSON('<==', payload);
    socket.write(JSON.stringify(payload));
  };

  const sendNotification = (method, params) => {
    sendJSON({
      jsonrpc: '2.0',
      method,
      params
    });
  };

  const context = new _ClientContext.default({
    log,
    env,
    notify: sendNotification,
    socket,
    vaults
  });
  (0, _oboe.default)(socket).on('done', async value => {
    logJSON('==>', value);
    const reply = await handleMessage(context, value);

    if (reply != null) {
      sendJSON(reply);
    }
  }).on('fail', () => {
    sendJSON({
      jsonrpc: '2.0',
      error: (0, _rpcError.parseError)().toObject()
    });
  });
  socket.on('end', async () => {
    log('disconnected');
    await context.clear();
    log('context cleared');
  });
  log('connected');
};

exports.default = _default;