"use strict";

exports.__esModule = true;
exports.isListening = exports.stop = exports.start = void 0;

var _config = require("@mainframe/config");

var _debug = _interopRequireDefault(require("debug"));

var _ServerHandler = _interopRequireDefault(require("./ServerHandler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _debug.default)('mainframe:daemon:rpc');
const servers = {};

const start = async envName => {
  log('start server using environment', envName);
  let server = servers[envName];

  if (server == null) {
    const env = _config.Environment.load(envName);

    server = new _ServerHandler.default(env);
    servers[envName] = server;
  }

  try {
    await server.start(); // eslint-disable-next-line no-console

    console.log(`${envName} server started`);
  } catch (err) {
    // eslint-disable-next-line no-console
    console.error(`${envName} server failed to start:`, err.message);
  }
};

exports.start = start;

const stop = async envName => {
  log('stop server using environment', envName);
  const server = servers[envName];

  if (server != null) {
    await server.stop();
  }
};

exports.stop = stop;

const isListening = envName => {
  const server = servers[envName];
  return server == null ? false : server.listening;
};

exports.isListening = isListening;