"use strict";

exports.__esModule = true;
exports.setPermissionsRequirements = exports.setPermission = exports.setUserPermissionsSettings = exports.setUserDefaultWallet = exports.remove = exports.publish = exports.open = exports.loadManifest = exports.install = exports.getManifestData = exports.getAll = exports.create = exports.close = exports.checkPermission = void 0;

var _appManifest = require("@mainframe/app-manifest");

var _client = require("@mainframe/client");

var _utilsId = require("@mainframe/utils-id");

var _AppsRepository = require("../../app/AppsRepository");

var _OwnApp = _interopRequireDefault(require("../../app/OwnApp"));

var _errors = require("../errors");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const createClientSession = (ctx, appID, userID, session) => {
  const app = ctx.openVault.apps.getAnyByID(appID);

  if (app == null) {
    throw (0, _errors.clientError)('Invalid appID');
  }

  const user = ctx.openVault.identities.getOwnUser(userID);

  if (user == null) {
    throw (0, _errors.clientError)('Invalid userID');
  }

  let defaultEthAccount = app.getDefaultEthAccount(userID) || ctx.queries.getUserDefaultEthAccount(userID);

  if (!defaultEthAccount) {
    // TODO: add a setting for default eth account on identity
    const ethWallet = ctx.openVault.wallets.getFirstEthWallet();

    if (ethWallet) {
      defaultEthAccount = ethWallet.getAccounts()[0];
    }
  }

  const appData = app instanceof _OwnApp.default ? {
    appID: (0, _client.idType)(appID),
    manifest: ctx.queries.getAppManifestData(appID),
    contentsPath: app.contentsPath
  } : {
    appID: (0, _client.idType)(appID),
    manifest: app.manifest,
    contentsPath: (0, _AppsRepository.getContentsPath)(ctx.env, app.manifest)
  };
  const wallets = ctx.queries.getUserEthWallets(userID);
  return {
    user: {
      id: (0, _client.idType)(userID),
      localID: user.localID,
      profile: user.profile,
      ethWallets: wallets
    },
    session: {
      sessID: (0, _client.idType)(session.sessID),
      permissions: session.permissions
    },
    defaultEthAccount,
    app: appData,
    isDev: session.isDev
  };
};

const checkPermission = {
  params: _client.APP_CHECK_PERMISSION_SCHEMA,
  handler: (ctx, params) => {
    const session = ctx.openVault.getSession((0, _utilsId.idType)(params.sessID));

    if (session == null) {
      throw (0, _errors.clientError)('Invalid session');
    }

    return {
      result: session.checkPermission(params.key, params.input)
    };
  }
};
exports.checkPermission = checkPermission;
const close = {
  params: _client.APP_CLOSE_SCHEMA,
  handler: (ctx, params) => {
    ctx.openVault.closeApp((0, _utilsId.idType)(params.sessID));
  }
};
exports.close = close;
const create = {
  params: _client.APP_CREATE_SCHEMA,
  handler: async (ctx, params) => {
    const {
      developerID
    } = params,
          data = _objectWithoutProperties(params, ["developerID"]);

    const app = await ctx.mutations.createApp(_objectSpread({}, data, {
      developerID: (0, _utilsId.idType)(developerID)
    }));
    return {
      appID: (0, _client.idType)(app.id)
    };
  }
};
exports.create = create;

const getAll = ctx => {
  const {
    apps,
    ownApps
  } = ctx.openVault.apps;

  const mapList = mapApps => {
    return Object.keys(mapApps).map(appID => {
      const app = mapApps[appID];
      const users = app.userIDs.reduce((acc, id) => {
        const user = ctx.openVault.identities.getOwnUser(id);

        if (user) {
          acc.push({
            id: (0, _client.idType)(id),
            profile: user.profile,
            settings: app.getSettings(id)
          });
        }

        return acc;
      }, []);
      return {
        localID: (0, _client.idType)(appID),
        name: app instanceof _OwnApp.default ? app.data.name : app.manifest.name,
        manifest: app.manifest || app._data,
        mfid: (0, _client.idType)(app.mfid),
        users: users
      };
    });
  };

  return {
    own: mapList(ownApps),
    installed: mapList(apps)
  };
};

exports.getAll = getAll;
const getManifestData = {
  params: _client.APP_GET_MANIFEST_DATA_SCHEMA,
  handler: async (ctx, params) => ({
    data: ctx.queries.getAppManifestData((0, _utilsId.idType)(params.appID), params.version)
  })
};
exports.getManifestData = getManifestData;
const install = {
  params: _client.APP_INSTALL_SCHEMA,
  handler: async (ctx, params) => {
    const app = await ctx.mutations.installApp({
      manifest: params.manifest,
      userID: (0, _utilsId.idType)(params.userID),
      permissionsSettings: params.permissionsSettings
    });
    return {
      appID: (0, _client.idType)(app.id),
      installationState: app.installationState
    };
  }
};
exports.install = install;
const loadManifest = {
  params: _client.APP_LOAD_MANIFEST_SCHEMA,
  handler: async (ctx, params) => {
    const res = await ctx.io.bzz.download(params.hash);
    const payload = await res.json();
    const manifest = (0, _appManifest.verifyManifest)(payload);
    const appID = ctx.openVault.apps.getID(manifest.id);
    const isOwn = appID != null && !!ctx.openVault.apps.getOwnByID(appID);
    return {
      manifest,
      isOwn,
      appID: appID ? (0, _client.idType)(appID) : undefined
    };
  }
};
exports.loadManifest = loadManifest;
const open = {
  params: _client.APP_OPEN_SCHEMA,
  handler: (ctx, params) => {
    const appID = (0, _utilsId.idType)(params.appID);
    const userID = (0, _utilsId.idType)(params.userID);
    const session = ctx.openVault.openApp(appID, userID);
    return createClientSession(ctx, appID, userID, session);
  }
};
exports.open = open;
const publish = {
  params: _client.APP_PUBLISH_SCHEMA,
  handler: async (ctx, params) => {
    const hash = await ctx.mutations.publishApp({
      appID: (0, _utilsId.idType)(params.appID),
      version: params.version
    });
    return {
      hash
    };
  }
};
exports.publish = publish;
const remove = {
  params: _client.APP_REMOVE_SCHEMA,
  handler: async (ctx, params) => {
    ctx.openVault.removeApp((0, _utilsId.idType)(params.appID));
    await ctx.openVault.save();
  }
};
exports.remove = remove;
const setUserDefaultWallet = {
  params: _client.APP_SET_USER_DEFAULT_WALLET_SCHEMA,
  handler: async (ctx, params) => {
    const appID = (0, _utilsId.idType)(params.appID);
    const userID = (0, _utilsId.idType)(params.userID);
    await ctx.mutations.setAppUserDefaultWallet(appID, userID, params.address);
  }
};
exports.setUserDefaultWallet = setUserDefaultWallet;
const setUserPermissionsSettings = {
  params: _client.APP_SET_USER_PERMISSIONS_SETTINGS_SCHEMA,
  handler: async (ctx, params) => {
    const appID = (0, _utilsId.idType)(params.appID);
    const userID = (0, _utilsId.idType)(params.userID);
    ctx.openVault.setAppUserPermissionsSettings(appID, userID, params.settings);
    await ctx.openVault.save();
  }
};
exports.setUserPermissionsSettings = setUserPermissionsSettings;
const setPermission = {
  params: _client.APP_SET_PERMISSION_SCHEMA,
  handler: async (ctx, params) => {
    const session = ctx.openVault.getSession((0, _utilsId.idType)(params.sessID));

    if (session == null) {
      throw (0, _errors.clientError)('Invalid session');
    }

    session.setPermission(params.key, params.value);
    const app = ctx.openVault.apps.getByID(session.appID);

    if (app == null) {
      throw (0, _errors.sessionError)('Invalid app');
    }

    app.setPermission(session.userID, params.key, params.value);
    await ctx.openVault.save();
  }
};
exports.setPermission = setPermission;
const setPermissionsRequirements = {
  params: _client.APP_SET_PERMISSIONS_REQUIREMENTS_SCHEMA,
  handler: async (ctx, params) => {
    const app = ctx.openVault.apps.getOwnByID((0, _utilsId.idType)(params.appID));

    if (app == null) {
      throw new Error('App not found');
    }

    app.setPermissionsRequirements(params.permissions, params.version);
    await ctx.openVault.save();
  }
};
exports.setPermissionsRequirements = setPermissionsRequirements;