"use strict";

exports.__esModule = true;
exports.getSubscribable = exports.subscribe = exports.publish = void 0;

var _client = require("@mainframe/client");

var _utilsId = require("@mainframe/utils-id");

var _feed = require("../../swarm/feed");

var _ContextSubscriptions = require("../../context/ContextSubscriptions");

const MINUTE = 60 * 1000;

class CommsSubscription extends _ContextSubscriptions.ContextSubscription {
  constructor() {
    super('comms_subscription');
  }

  startNotify(notify, observable) {
    this.data = observable.subscribe(result => notify(this.method, {
      subscription: this.id,
      result
    }));
  }

  async dispose() {
    if (this.data != null) {
      this.data.unsubscribe();
    }
  }

}

const publish = {
  params: _client.COMMS_PUBLISH_SCHEMA,
  handler: async (ctx, params) => {
    const localID = ctx.queries.getContactLocalIDByAppApprovedID((0, _utilsId.idType)(params.appID), (0, _utilsId.idType)(params.userID), (0, _utilsId.idType)(params.contactID));

    if (!localID) {
      throw new Error('Contact not found');
    }

    await ctx.mutations.publishAppData((0, _utilsId.idType)(params.appID), localID, params.key, params.value);
  }
};
exports.publish = publish;
const subscribe = {
  params: _client.COMMS_SUBSCRIBE_SCHEMA,
  handler: async (ctx, params) => {
    const localID = ctx.queries.getContactLocalIDByAppApprovedID((0, _utilsId.idType)(params.appID), (0, _utilsId.idType)(params.userID), (0, _utilsId.idType)(params.contactID));

    if (!localID) {
      throw new Error('Contact not found');
    }

    const appFeeds = await ctx.queries.getContactAppFeeds((0, _utilsId.idType)(params.appID), localID);

    if (!appFeeds[params.key]) {
      throw new Error('Feed not found');
    }

    const sub = new CommsSubscription();
    ctx.subscriptions.set(sub); // TODO: Integrate with FeedsHandler to de-duplicate multiple calls to poll the same feed

    const poll = (0, _feed.pollFeedJSON)(ctx.io.bzz, appFeeds[params.key].feedHash, {
      interval: MINUTE
    });
    sub.startNotify(ctx.notify, poll);
    return sub.id;
  }
};
exports.subscribe = subscribe;
const getSubscribable = {
  params: _client.COMMS_GET_SUBSCRIBABLE_SCHEMA,
  handler: async (ctx, params) => {
    const localID = ctx.queries.getContactLocalIDByAppApprovedID((0, _utilsId.idType)(params.appID), (0, _utilsId.idType)(params.userID), (0, _utilsId.idType)(params.contactID));

    if (!localID) {
      throw new Error('Contact not found');
    }

    const appFeeds = await ctx.queries.getContactAppFeeds((0, _utilsId.idType)(params.appID), localID);
    return Object.keys(appFeeds).filter(key => appFeeds[key].type === 'json');
  }
};
exports.getSubscribable = getSubscribable;