"use strict";

exports.__esModule = true;
exports.deleteContact = exports.createContactFromFeed = exports.createContactFromPeer = exports.getPeers = exports.addPeer = exports.addPeerByFeed = exports.unlinkEthWallet = exports.linkEthWallet = exports.updateUser = exports.getOwnUsers = exports.getOwnDevelopers = exports.createUser = exports.createDeveloper = void 0;

var _client = require("@mainframe/client");

var _utilsId = require("@mainframe/utils-id");

const createDeveloper = {
  params: _client.IDENTITY_CREATE_OWN_DEVELOPER_SCHEMA,
  handler: async (ctx, params) => {
    const dev = await ctx.mutations.createDeveloper(params.profile);
    return {
      id: (0, _client.idType)(dev.localID)
    };
  }
};
exports.createDeveloper = createDeveloper;
const createUser = {
  params: _client.IDENTITY_CREATE_OWN_USER_SCHEMA,
  handler: async (ctx, params) => {
    const user = await ctx.mutations.createUser(params.profile);
    return {
      id: (0, _client.idType)(user.localID)
    };
  }
};
exports.createUser = createUser;

const getOwnDevelopers = ctx => {
  const {
    ownDevelopers
  } = ctx.openVault.identities;
  const developers = Object.keys(ownDevelopers).map(id => {
    const wallets = ctx.queries.getUserEthWallets(id);
    return {
      id: ownDevelopers[id].id,
      localID: ownDevelopers[id].localID,
      profile: ownDevelopers[id].profile,
      ethWallets: wallets
    };
  });
  return {
    developers
  };
};

exports.getOwnDevelopers = getOwnDevelopers;

const getOwnUsers = ctx => {
  const {
    ownUsers
  } = ctx.openVault.identities;
  const users = Object.keys(ownUsers).map(id => {
    const wallets = ctx.queries.getUserEthWallets(id);
    return {
      id: ownUsers[id].id,
      localID: id,
      profile: ownUsers[id].profile,
      feedHash: ownUsers[id].publicFeed.feedHash,
      ethWallets: wallets
    };
  });
  return {
    users
  };
};

exports.getOwnUsers = getOwnUsers;
const updateUser = {
  params: _client.IDENTITY_UPDATE_USER_SCHEMA,
  handler: async (ctx, params) => {
    await ctx.mutations.updateUser((0, _utilsId.idType)(params.userID), params.profile);
  }
};
exports.updateUser = updateUser;
const linkEthWallet = {
  params: _client.IDENTITY_LINK_ETH_WALLET_SCHEMA,
  handler: async (ctx, params) => {
    if (!ctx.openVault.wallets.getEthWalletByID(params.walletID)) {
      throw new Error('Wallet not found');
    }

    if (!ctx.openVault.identities.getOwn((0, _utilsId.idType)(params.id))) {
      throw new Error('Identity not found');
    }

    ctx.openVault.identityWallets.linkWalletToIdentity(params.id, params.walletID, params.address);
    await ctx.openVault.save();
  }
};
exports.linkEthWallet = linkEthWallet;
const unlinkEthWallet = {
  params: _client.IDENTITY_UNLINK_ETH_WALLET_SCHEMA,
  handler: async (ctx, params) => {
    ctx.openVault.identityWallets.unlinkWalletToIdentity(params.id, params.walletID, params.address);
    await ctx.openVault.save();
  }
};
exports.unlinkEthWallet = unlinkEthWallet;
const addPeerByFeed = {
  params: _client.IDENTITY_ADD_PEER_BY_FEED_SCHEMA,
  handler: async (ctx, params) => {
    const peer = await ctx.mutations.addPeerByFeed(params.feedHash);
    return {
      id: (0, _client.idType)(peer.localID)
    };
  }
};
exports.addPeerByFeed = addPeerByFeed;
const addPeer = {
  params: _client.IDENTITY_ADD_PEER_SCHEMA,
  handler: async (ctx, params) => {
    const peer = await ctx.mutations.addPeer(params);
    return {
      id: (0, _client.idType)(peer.localID)
    };
  }
};
exports.addPeer = addPeer;

const getPeers = ctx => {
  const peerUsers = ctx.openVault.identities.peerUsers;
  const peers = Object.keys(peerUsers).map(id => {
    const peer = peerUsers[id];
    return {
      id: (0, _client.idType)(id),
      publicFeed: peer.publicFeed,
      profile: {
        name: peer.name,
        avatar: peer.avatar
      }
    };
  });
  return {
    peers
  };
};

exports.getPeers = getPeers;
const createContactFromPeer = {
  params: _client.IDENTITY_CREATE_CONTACT_FROM_PEER_SCHEMA,
  handler: async (ctx, params) => {
    const contact = await ctx.mutations.createContactFromPeer((0, _utilsId.idType)(params.userID), (0, _utilsId.idType)(params.peerID));
    return {
      id: (0, _client.idType)(contact.localID)
    };
  }
};
exports.createContactFromPeer = createContactFromPeer;
const createContactFromFeed = {
  params: _client.IDENTITY_CREATE_CONTACT_FROM_FEED_SCHEMA,
  handler: async (ctx, params) => {
    const contact = await ctx.mutations.createContactFromFeed((0, _utilsId.idType)(params.userID), params.feedHash);
    return {
      id: (0, _client.idType)(contact.localID)
    };
  }
};
exports.createContactFromFeed = createContactFromFeed;
const deleteContact = {
  params: _client.IDENTITY_DELETE_CONTACT_SCHEMA,
  handler: async (ctx, params) => {
    await ctx.mutations.deleteContact((0, _utilsId.idType)(params.userID), (0, _utilsId.idType)(params.contactID));
  }
};
exports.deleteContact = deleteContact;