"use strict";

exports.__esModule = true;
exports.setUsersDefaultWallet = exports.addLedgerEthAccounts = exports.getLedgerEthAccounts = exports.signTransaction = exports.getUserEthAccounts = exports.getUserEthWallets = exports.deleteWallet = exports.addHDAccount = exports.importMnemonic = exports.createHDWallet = void 0;

var _client = require("@mainframe/client");

var _ledgerClient = require("../../wallet/ledgerClient");

const createHDWallet = {
  params: _client.WALLET_CREATE_HD_SCHEMA,
  handler: async (ctx, params) => {
    // $FlowFixMe Issue with promise type
    const hdWallet = await ctx.mutations.createHDWallet(params.blockchain, params.name, params.userID);
    return {
      localID: hdWallet.localID,
      mnemonic: hdWallet.mnemonic,
      accounts: hdWallet.getAccounts()
    };
  }
};
exports.createHDWallet = createHDWallet;
const importMnemonic = {
  params: _client.WALLET_IMPORT_MNEMONIC_SCHEMA,
  handler: async (ctx, params) => {
    // $FlowFixMe Issue with promise type
    const hdWallet = await ctx.mutations.importHDWallet(params);
    return {
      localID: hdWallet.localID,
      accounts: hdWallet.getAccounts()
    };
  }
};
exports.importMnemonic = importMnemonic;
const addHDAccount = {
  params: _client.WALLET_ADD_HD_ACCOUNT_SCHEMA,
  handler: async (ctx, params) => {
    return ctx.mutations.addHDWalletAccount(params);
  }
};
exports.addHDAccount = addHDAccount;
const deleteWallet = {
  params: _client.WALLET_DELETE_SCHEMA,
  handler: async (ctx, params) => {
    await ctx.mutations.deleteWallet(params.chain, params.type, params.localID);
  }
};
exports.deleteWallet = deleteWallet;
const getUserEthWallets = {
  params: _client.WALLET_GET_USER_ETH_WALLETS_SCHEMA,
  handler: async (ctx, params) => {
    const wallets = ctx.queries.getUserEthWallets(params.userID);
    return {
      hd: wallets.filter(w => w.type === 'hd'),
      ledger: wallets.filter(w => w.type === 'ledger')
    };
  }
};
exports.getUserEthWallets = getUserEthWallets;
const getUserEthAccounts = {
  params: _client.WALLET_GET_USER_ETH_ACCOUNTS_SCHEMA,
  handler: async (ctx, params) => {
    const accounts = ctx.queries.getUserEthAccounts(params.userID);
    return accounts;
  }
};
exports.getUserEthAccounts = getUserEthAccounts;
const signTransaction = {
  params: _client.WALLET_SIGN_TRANSACTION_SCHEMA,
  handler: async (ctx, params) => {
    return ctx.openVault.wallets.signTransaction(params.chain, params.transactionData);
  }
};
exports.signTransaction = signTransaction;
const getLedgerEthAccounts = {
  params: _client.WALLET_GET_LEDGER_ETH_ACCOUNTS_SCHEMA,
  handler: async (ctx, params) => {
    return (0, _ledgerClient.getAccountsByPage)(params);
  }
};
exports.getLedgerEthAccounts = getLedgerEthAccounts;
const addLedgerEthAccounts = {
  params: _client.WALLET_ADD_LEDGER_ETH_ACCOUNT_SCHEMA,
  handler: async (ctx, params) => {
    return ctx.mutations.addLedgerWalletAccounts(params.indexes, params.name, params.userID);
  }
};
exports.addLedgerEthAccounts = addLedgerEthAccounts;
const setUsersDefaultWallet = {
  params: _client.WALLET_SET_USER_DEFAULT_SCHEMA,
  handler: async (ctx, params) => {
    return ctx.mutations.setUsersDefaultWallet(params.userID, params.address);
  }
};
exports.setUsersDefaultWallet = setUsersDefaultWallet;