"use strict";

exports.__esModule = true;
exports.default = void 0;

var _Vault = _interopRequireDefault(require("./Vault"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class VaultRegistry {
  constructor() {
    _defineProperty(this, "_socketByVaultPath", new Map());

    _defineProperty(this, "_vaultBySocket", new WeakMap());
  }

  getVault(socket) {
    return this._vaultBySocket.get(socket);
  }

  isLocked(path) {
    const socket = this._socketByVaultPath.get(path);

    if (socket == null) {
      return false;
    }

    const existing = this.getVault(socket);

    if (existing == null) {
      // Socket no longer exists, remove lock
      this._socketByVaultPath.delete(path);

      return false;
    }

    return true;
  }

  async close(socket) {
    const vault = this.getVault(socket);

    if (vault != null) {
      await vault.save();

      this._socketByVaultPath.delete(vault.path);

      this._vaultBySocket.delete(socket);
    }
  }

  async create(socket, path, password) {
    if (this.isLocked(path)) {
      throw new Error('Vault is already open by another client');
    }

    const vault = await _Vault.default.create(path, password);

    this._socketByVaultPath.set(path, socket);

    this._vaultBySocket.set(socket, vault);

    return vault;
  }

  async open(socket, path, password) {
    if (this.isLocked(path)) {
      throw new Error('Vault is already open by another client');
    }

    const vault = await _Vault.default.open(path, password);

    this._socketByVaultPath.set(path, socket);

    this._vaultBySocket.set(socket, vault);

    return vault;
  }

}

exports.default = VaultRegistry;