"use strict";

exports.__esModule = true;
exports.default = void 0;

var _hdkey = _interopRequireDefault(require("ethereumjs-wallet/hdkey"));

var _ethSigUtil = _interopRequireDefault(require("eth-sig-util"));

var _bip = _interopRequireDefault(require("bip39"));

var _utilsId = require("@mainframe/utils-id");

var _ethereumjsTx = _interopRequireDefault(require("ethereumjs-tx"));

var _ethereumjsUtil = require("ethereumjs-util");

var _AbstractWallet = _interopRequireDefault(require("./AbstractWallet"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const HD_PATH_STRING = `m/44'/60'/0'/0`;

class HDWallet extends _AbstractWallet.default {
  constructor(params) {
    super(_objectSpread({}, params, {
      type: 'hd'
    }));

    _defineProperty(this, "_hdKey", void 0);

    _defineProperty(this, "_root", void 0);

    _defineProperty(this, "_mnemonic", void 0);

    _defineProperty(this, "_hdPath", void 0);

    _defineProperty(this, "_wallets", void 0);

    this._wallets = {};
    this._hdPath = params.hdPath ? params.hdPath : HD_PATH_STRING;

    this._initFromMnemonic(params.mnemonic);

    if (params.activeAccounts && params.activeAccounts.length) {
      this.addAccounts(params.activeAccounts.map(a => Number(a)));
    }

    if (!this._wallets.length) {
      this.addAccounts([0]);
    }
  } // Getters


  get publicExtendedKey() {
    return this._hdKey.publicExtendedKey();
  }

  get mnemonic() {
    return this._mnemonic;
  } // Internal


  _initFromMnemonic(mnemonic) {
    const seed = _bip.default.mnemonicToSeed(mnemonic);

    this._hdKey = _hdkey.default.fromMasterSeed(seed);
    this._root = this._hdKey.derivePath(this._hdPath);
    this._mnemonic = mnemonic;
  }

  _accountWalletByAddress(address) {
    const targetAddress = _ethSigUtil.default.normalize(address);

    let accountWallet;
    const indexes = Object.keys(this._wallets);
    indexes.forEach(i => {
      const normalizedAddr = _ethSigUtil.default.normalize(this._wallets[i].getAddress().toString('hex'));

      if (normalizedAddr === targetAddress) {
        accountWallet = this._wallets[i];
      }
    });
    return accountWallet;
  } // Public


  getAccounts() {
    // $FlowFixMe mapping type
    return Object.keys(this._wallets).map(i => {
      const wallet = this._wallets[i];
      return _ethSigUtil.default.normalize(wallet.getAddress().toString('hex'));
    });
  }

  addAccounts(indexes) {
    const newWallets = [];
    indexes.forEach(i => {
      if (!this._wallets[String(i)]) {
        const child = this._root.deriveChild(i);

        const wallet = child.getWallet();
        newWallets.push(wallet);
        this._wallets[String(i)] = wallet;
      }
    });
    const hexWallets = newWallets.map(w => {
      return _ethSigUtil.default.normalize(w.getAddress().toString('hex'));
    });
    return hexWallets;
  }

  containsAccount(account) {
    return !!this._accountWalletByAddress(account);
  }

  signTransaction(params) {
    const accountWallet = this._accountWalletByAddress(params.from);

    if (!accountWallet) {
      throw new Error('Invalid from address');
    }

    const tx = new _ethereumjsTx.default(params);
    tx.sign(accountWallet.getPrivateKey());
    return (0, _ethereumjsUtil.addHexPrefix)(tx.serialize().toString('hex'));
  }

  sign(params) {
    const accountWallet = this._accountWalletByAddress(params.address);

    if (!accountWallet) {
      throw new Error('Invalid address');
    }

    return (0, _ethereumjsUtil.addHexPrefix)(_ethSigUtil.default.personalSign(accountWallet.getPrivateKey(), {
      data: params.data
    }));
  }

}

exports.default = HDWallet;

_defineProperty(HDWallet, "create", name => {
  const params = {
    name,
    localID: (0, _utilsId.uniqueID)(),
    mnemonic: _bip.default.generateMnemonic(),
    hdPath: HD_PATH_STRING,
    activeAccounts: ['0']
  };
  return new HDWallet(params);
});

_defineProperty(HDWallet, "import", params => {
  const wallet = new HDWallet(_objectSpread({}, params, {
    localID: (0, _utilsId.uniqueID)(),
    activeAccounts: ['0']
  }));
  return wallet;
});

_defineProperty(HDWallet, "fromJSON", params => new HDWallet(params));

_defineProperty(HDWallet, "toJSON", wallet => ({
  mnemonic: wallet._mnemonic,
  localID: wallet._localID,
  activeAccounts: Object.keys(wallet._wallets),
  name: wallet.name
}));