"use strict";

exports.__esModule = true;
exports.default = void 0;

var _ethereumjsTx = _interopRequireDefault(require("ethereumjs-tx"));

var _ethereumjsUtil = _interopRequireDefault(require("ethereumjs-util"));

var _web3Utils = require("web3-utils");

var _ledgerClient = require("./ledgerClient");

var _AbstractWallet = _interopRequireDefault(require("./AbstractWallet"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class LedgerWallet extends _AbstractWallet.default {
  // $FlowFixMe: Wallet type
  // Store address at 0 to identify ledger
  constructor(params) {
    super(_objectSpread({}, params, {
      type: 'ledger'
    }));

    _defineProperty(this, "_firstAddress", void 0);

    _defineProperty(this, "_activeAccounts", void 0);

    this._activeAccounts = params.activeAccounts;
    this._firstAddress = params.firstAddress;
  } // Getters


  get firstAddress() {
    return this._firstAddress;
  }

  get activeAccounts() {
    return this._activeAccounts;
  } // Public


  containsAccount(account) {
    return Object.values(this._activeAccounts).includes((0, _web3Utils.toChecksumAddress)(account));
  }

  getAccounts() {
    // $FlowFixMe mixed values are actually strings
    return Object.values(this._activeAccounts);
  }

  getIndexForAccount(account) {
    return Object.keys(this._activeAccounts).find(i => {
      return this._activeAccounts[i] === (0, _web3Utils.toChecksumAddress)(account);
    });
  }

  async addAccounts(indexes) {
    const newAddresses = [];

    for (let i = 0; i < indexes.length; i++) {
      const index = indexes[i];
      const stringIndex = String(index);

      if (!this._activeAccounts[stringIndex]) {
        const address = await (0, _ledgerClient.getAddressAtIndex)({
          index: index
        });
        this._activeAccounts[stringIndex] = address;
        newAddresses.push(address);
      }
    }

    return newAddresses;
  }

  async signTransaction(params) {
    if (!params.chainId) {
      throw new Error('ethereum chain id not set in tx params');
    }

    const index = this.getIndexForAccount(params.from);

    if (!index) {
      throw new Error('account not registered with this device');
    }

    const tx = new _ethereumjsTx.default(params);
    tx.v = _ethereumjsUtil.default.bufferToHex(tx.getChainId());
    tx.r = '0x00';
    tx.s = '0x00';
    const txHex = tx.serialize().toString('hex');
    const res = await (0, _ledgerClient.signTransaction)(Number(index), txHex);
    tx.v = Buffer.from(res.v, 'hex');
    tx.r = Buffer.from(res.r, 'hex');
    tx.s = Buffer.from(res.s, 'hex');
    const valid = tx.verifySignature();

    if (valid) {
      return _ethereumjsUtil.default.addHexPrefix(tx.serialize().toString('hex'));
    } else {
      throw new Error('invalid transaction signature');
    }
  }

  sign(params) {
    params; // TODO: Needs implementing
  }

}

exports.default = LedgerWallet;

_defineProperty(LedgerWallet, "fromJSON", params => new LedgerWallet(params));

_defineProperty(LedgerWallet, "toJSON", wallet => ({
  activeAccounts: wallet.activeAccounts,
  localID: wallet.localID,
  firstAddress: wallet.firstAddress,
  name: wallet.name
}));