"use strict";

exports.__esModule = true;
exports.signTransaction = exports.getAccountsByPage = exports.getAccounts = exports.getAddressAtIndex = void 0;

var _hwTransportNodeHid = _interopRequireDefault(require("@ledgerhq/hw-transport-node-hid"));

var _hwAppEth = _interopRequireDefault(require("@ledgerhq/hw-app-eth"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ITEMS_PER_PAGE = 10;
const HD_PATH = "m/44'/60'/0'/";
let ledgerApp;

const getLedgerApp = async () => {
  if (ledgerApp) {
    return ledgerApp;
  }

  const transport = await _hwTransportNodeHid.default.create();
  ledgerApp = new _hwAppEth.default(transport);
  return ledgerApp;
};

const getAddressAtIndex = async params => {
  const app = await getLedgerApp();
  const path = HD_PATH + params.index;
  const account = await app.getAddress(path);
  return account.address;
};

exports.getAddressAtIndex = getAddressAtIndex;

const getAccounts = async params => {
  const app = await getLedgerApp();
  const accounts = [];

  for (let i = params.from; i < params.to; i++) {
    const path = HD_PATH + i;
    const account = await app.getAddress(path);
    accounts.push(account);
  }

  return accounts.map(a => a.address);
};

exports.getAccounts = getAccounts;

const getAccountsByPage = async params => {
  const from = (params.pageNum - 1) * ITEMS_PER_PAGE;
  const to = from + ITEMS_PER_PAGE;
  const accounts = await getAccounts({
    from,
    to
  });
  return accounts;
};

exports.getAccountsByPage = getAccountsByPage;

const signTransaction = async (accountIndex, txHex) => {
  const app = await getLedgerApp();
  const path = HD_PATH + accountIndex;
  return app.signTransaction(path, txHex);
};

exports.signTransaction = signTransaction;